<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-10-27 23:51:06
 * website: https://loginbasicoemo.com
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') or exit('No direct script access allowed');

class Admin extends CI_Controller
{
    private $user, $menu, $subMenu;

    public function __construct()
    {

        parent::__construct();
        is_logged_in();

        // Obtener los datos del usuario desde la sesión
        $this->user = $this->session->userdata();

        // Verificar si los datos del usuario están disponibles
        if (empty($this->user)) {
            redirect('auth'); // Redirige al login si no hay datos de usuario
        }

        if ($this->user['role_id'] != 1) {
            // Redirigir a la página por defecto del usuario
            redirect($this->session->userdata('redirection'));
        }

        // Obtener el email del usuario
        $email = $this->user['email'];
        $this->load->model('User_model');

        // Obtener el idioma del usuario desde la base de datos

        $default_language = $this->User_model->get_user_language($email);

        $friendly_language = $default_language;

        // Mapeo de nombres amigables a códigos de idioma estándar
        $this->load->config('language');
        $language_directory      = $this->config->item('language_directory');
        $language_directory_name = $this->config->item('language_directory_name');

        // Obtener el nombre del directorio para el idioma
        $language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

        // Determinar el código de idioma
        $this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

        // Cargar el archivo de idioma desde el directorio correcto
        $this->lang->load('aplicacion_lang', $language_directory_name_value);

        $this->load->model('Config_model');
        $this->load->library('grocery_CRUD');
        $this->user    = $this->User_model->getUser($this->session->userdata('email'));
        $this->menu    = $this->User_model->getSideMenu($this->session->userdata('role_id'));
        $this->subMenu = $this->User_model->getSideSubMenu($this->menu);

        $this->load->model('Seo_model');                           // Cargar el modelo SEO
        $this->data['seo'] = $this->Seo_model->get_seo_settings(); // Obtener configuraciones SEO

        // Verificar si la clase Form_validation ya está cargada
        if (! isset($this->form_validation)) {
            $this->load->library('form_validation');
        }
        $this->load->config('app_version'); // Cargar configuración de la versión
                                            // Cargar la librería de Zip y la utilidad de base de datos
        $this->load->library('zip');
        $this->load->dbutil();
                                            // Aumentar el límite de memoria y tiempo de ejecución
        ini_set('memory_limit', '512M');    // 512 MB o más, según sea necesario
        ini_set('max_execution_time', 300); // 300 segundos = 5 minutos

        //para cargar plugins
        $this->load->library('HookManager');

    }

    public function index()
    {
        $installed_plugins = $this->get_installed_plugins();
        $data['title']     = $this->lang->line('Dashboard');
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista

        $data['lang_code'] = $this->lang_code; // Código del idioma

        $data['maintenance_mode'] = $this->config->item('maintenance_mode');

        $data['totalusers']        = $this->User_model->getNumberOfUsers();
        $data['totalactiveusers']  = $this->User_model->getNumberOfActiveUsers();
        $data['totalonlineusers']  = $this->User_model->get_total_online_users();
        $data['installed_plugins'] = $installed_plugins; // Pasar los plugins instalados a la vista

        $data['log_content'] = $this->get_last_log(); // Pasar el contenido del log a la vista

        // Obtener la versión actual de la configuración
        $current_version = $this->config->item('app_version');

        // Pasar la versión actual a la vista
        $data['app_version'] = $current_version;

        // Cargar todos los plugins
        $this->hookmanager->load_plugins();

        // Capturar la salida del hook admin_index
        $data['plugin_outputs'] = $this->hookmanager->run_hooks('admin_index');

        $this->actualizar_ultima_actividad();

        $this->load->view('templates/header', $data);
        $this->load->view
            ('templates/sidebar', $data);
        $this->load->view
            ('templates/topbar', $data);
        $this->load->view('admin/index', $data);
        $this->load->view('templates/footer');
    }

    // Método para leer el último archivo de log

    /**
     * Gets the last log.
     *
     * @return     string  The last log.
     */
    public function get_last_log()
    {
        $log_path      = APPPATH . 'logs/';                           // Ruta a la carpeta de logs
        $logs          = array_diff(scandir($log_path), ['..', '.']); // Obtener todos los archivos de log
        $last_log_file = end($logs);                                  // Obtener el último archivo de log

        if ($last_log_file) {
            $log_content = file_get_contents($log_path . $last_log_file); // Leer el contenido del último archivo de log
            return $log_content;
        } else {
            return 'No logs found.';
        }
    }

    public function generate_key_form()
    {

        $data['title']     = $this->lang->line('generate_api_key_title');
        $data['user']      = $this->user;
        $data['users']     = $this->User_model->get_users(); // Obtener usuarios desde el modelo
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma
                                                    // Cargar cualquier mensaje de la sesión
        $data['message'] = $this->session->flashdata('message');
        $data['error']   = $this->session->flashdata('error');

        // Obtener todas las claves API y nombres de usuarios ordenadas por id de clave de forma descendente
        $this->db->select('keys.id, keys.key, user.name as user_name');
        $this->db->from('keys');
        $this->db->join('user', 'keys.user_id = user.id');
        $this->db->order_by('keys.id', 'ASC'); // Ordenar por id de clave de forma descendente
        $data['api_keys'] = $this->db->get()->result_array();

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/generate_api_key', $data);
        $this->load->view('templates/footer');
    }

    /**
     * delete Api key
     *
     * @param      <type>  $key_id  The key identifier
     */
    public function delete_key($key_id)
    {
        $this->db->delete('keys', ['id' => $key_id]);
        $this->session->set_flashdata('message', 'Clave API eliminada correctamente.');
        redirect('admin/generate_key_form');
    }

    /**
     * Generate Api key
     */
    public function generate_key_post()
    {
        $user_id = $this->input->post('user_id');

        if (empty($user_id)) {
            $this->session->set_flashdata('error', 'Por favor selecciona un usuario.');
        } else {
            $generated_key = $this->generate_api_key($user_id);

            if ($generated_key) {
                $this->session->set_flashdata('message', 'Clave API generada correctamente para el usuario ID: ' . $user_id);
            } else {
                $this->session->set_flashdata('error', 'Error al generar la clave API para el usuario ID: ' . $user_id);
            }
        }

        // Redirigir a la página de generación de clave con los datos actualizados
        redirect('admin/generate_key_form');
    }

    /**
     * Generate Api key for user_id
     *
     * @param      <type>  $user_id  The user identifier
     *
     * @return     bool    ( description_of_the_return_value )
     */
    private function generate_api_key($user_id)
    {
                                              // Genera una clave única (puedes ajustar la lógica según necesites)
        $api_key = bin2hex(random_bytes(20)); // Genera una cadena hexadecimal de 40 caracteres

        // Guarda la clave API en la base de datos
        $data = [
            'user_id'       => $user_id,
            'key'           => $api_key,
            'level'         => 1,
            'ignore_limits' => 0,
            'is_active'     => 1,
            'ip_addresses'  => null,
            'date_created'  => time(),
        ];

        $this->db->insert('keys', $data);

        // Verifica si la inserción fue exitosa
        if ($this->db->affected_rows() > 0) {
            return $api_key;
        } else {
            return false;
        }
    }

    /**
     * Update user activity
     */
    public function actualizar_ultima_actividad()
    {
        $email = $this->session->userdata('email'); // Obtener el email del usuario desde la sesión

        // Llamar al método del modelo para actualizar la última actividad
        $this->User_model->updateLastActivity($email);
    }

    /**
     * Change Language
     *
     * @throws     Exception  (description)
     */
    public function change_language()
    {
        // Obtener el nuevo idioma desde el formulario
        $language = $this->input->post('language');

        // ID fijo del usuario admin
        $admin_id = 15;

        try {
            // Cargar la base de datos
            $this->load->database();

            // Cambiar el valor por defecto del campo language en la tabla user
            $alter_table_sql = "ALTER TABLE `user`
                            MODIFY `language` varchar(20) CHARACTER SET utf8mb3 COLLATE utf8mb3_general_ci DEFAULT '$language';";
            if (! $this->db->query($alter_table_sql)) {
                throw new Exception('Error al modificar el campo language en la tabla user.');
            }

            // Actualizar el idioma del usuario admin
            $this->db->set('language', $language);
            $this->db->where('id', $admin_id);
            if (! $this->db->update('user')) {
                throw new Exception('Error al actualizar el idioma del usuario admin.');
            }

            // Ruta del archivo de configuración
            $config_file = APPPATH . 'config/config.php';

            // Cargar el contenido del archivo de configuración
            if (! $config_content = @file_get_contents($config_file)) {
                throw new Exception('Error al leer el archivo de configuración.');
            }

            // Buscar y reemplazar la configuración de idioma
            $new_config_content = preg_replace(
                '/\$config\[\'language\'\]\s+=\s+[\'"].+[\'"];/',
                "\$config['language'] = '$language';",
                $config_content
            );

            // Escribir el nuevo contenido en el archivo de configuración
            if (@file_put_contents($config_file, $new_config_content) === false) {
                throw new Exception('Error al escribir en el archivo de configuración.');
            }

            // Redirigir
            redirect('admin/ajustes');
        } catch (Exception $e) {
            // Manejo de errores
            log_message('error', $e->getMessage());
            show_error('Se produjo un error al cambiar el idioma. Por favor, intente de nuevo más tarde.');
        }
    }

    /**
     * Saves a timezone.
     */
    public function save_timezone()
    {
        $timezone = $this->input->post('timezone');

        // Validar la zona horaria seleccionada
        if (in_array($timezone, timezone_identifiers_list())) {
            // Ruta del archivo de configuración
            $config_file = APPPATH . 'config/config.php';

            // Cargar el contenido del archivo de configuración
            $config_content = file_get_contents($config_file);

            // Buscar y reemplazar la configuración de zona horaria
            $new_config_content = preg_replace(
                '/\$config\[\'time_reference\'\]\s+=\s+[\'"].+[\'"];/',
                "\$config['time_reference'] = '$timezone';",
                $config_content
            );

            // Escribir el nuevo contenido en el archivo de configuración
            file_put_contents($config_file, $new_config_content);

            // Redirigir a la página deseada después de guardar
            redirect('admin/ajustes');
        } else {
            // Manejar error si se selecciona una zona horaria no válida
            echo 'Zona horaria no válida';
        }
    }

    public function ajustes()
    {

        $this->actualizar_ultima_actividad();
        $this->hookmanager->load_plugins();
        $data['tools_plugin_outputs'] = $this->hookmanager->run_hooks('ajustes_tools');
        $data['selected_design_id']   = $this->get_selected_design_id(); // Pasar el ID del diseño seleccionado a la vista
        $data['title']                = $this->lang->line('APP Settings');
        $data['user']                 = $this->user;
        $data['menu']                 = $this->menu;
        $data['subMenu']              = $this->subMenu;
        $data['lang']                 = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code']            = $this->lang_code;      // Código del idioma

        // Obtener la configuración de idiomas
        $this->load->config('language');
        $data['language_directory']      = $this->config->item('language_directory');
        $data['language_directory_name'] = $this->config->item('language_directory_name');

        $data['activar_registro']     = $this->Config_model->get_config_value('activar_registro');['value'];
        $data['activar_menu_lateral'] = $this->Config_model->get_config_value('activar_menu_lateral');
        $data['default_activation']   = $this->Config_model->get_config_value('default_activation');

        $data['gradiente']       = $this->Config_model->get_config_value('gradiente')['value'];
        $data['gradiente2']      = $this->Config_model->get_config_value('gradiente2')['value'];
        $data['gradiente3']      = $this->Config_model->get_config_value('gradiente3')['value'];
        $data['page_background'] = $this->Config_model->get_config_value('page_background')['value'];

        $data['background_container'] = $this->Config_model->get_config_value('background_container')['value'];
        $data['text_header']          = $this->Config_model->get_config_value('text_header')['value'];

        $data['button_color'] = $this->Config_model->get_config_value('button_color')['value'];
        // Recuperar la fuente actual desde la base de datos
        $data['current_font']        = $this->Config_model->get_config_value('font_family');
        $data['current_custom_font'] = $this->Config_model->get_config_value('custom_font');

        $data['email_input']      = $this->Config_model->get_config_value('email_input')['value'];
        $data['password_input']   = $this->Config_model->get_config_value('password_input')['value'];
        $data['checkbox_text']    = $this->Config_model->get_config_value('checkbox_text')['value'];
        $data['posicion']         = $this->Config_model->get_config_value('posicion')['value'];
        $data['forgot_create']    = $this->Config_model->get_config_value('forgot_create')['value'];
        $data['animation_effect'] = $this->Config_model->get_config_value('animation_effect')['value'];

        $data['texto_login']                = $this->Config_model->get_config_value('texto_login')['value'];
        $data['texto_email_placeholder']    = $this->Config_model->get_config_value('texto_email_placeholder')['value'];
        $data['texto_password_placeholder'] = $this->Config_model->get_config_value('texto_password_placeholder')['value'];
        $data['texto_terminos']             = $this->Config_model->get_config_value('texto_terminos')['value'];
        $data['texto_boton']                = $this->Config_model->get_config_value('texto_boton')['value'];
        $data['texto_forgot']               = $this->Config_model->get_config_value('texto_forgot')['value'];
        $data['texto_crear_cuenta']         = $this->Config_model->get_config_value('texto_crear_cuenta')['value'];

        $data['css_placeholder'] = $this->Config_model->get_config_value('css_placeholder')['value'];
        $data['css_focus']       = $this->Config_model->get_config_value('css_focus')['value'];
        $data['css_general']     = $this->Config_model->get_config_value('css_general')['value'];

        $data['contenido_terminos'] = $this->Config_model->get_config_value('contenido_terminos')['value'];

        $data['tipo_plantilla'] = $this->Config_model->get_config_value('tipo_plantilla')['value'];

        $contenido_sidebar_config  = $this->Config_model->get_config_value('contenido_sidebar');
        $data['contenido_sidebar'] = isset($contenido_sidebar_config['value']) ? $contenido_sidebar_config['value'] : ''; // 'normal' como valor por defecto

        $data['source_image'] = $this->Config_model->get_config_value('source_image')['value'];
        $data['footer']       = $this->Config_model->get_config_value('footer')['value'];

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/ajustes', $data);
        $this->load->view('templates/footer');
    }

    /**
     * Does an upload.
     */
    public function do_upload()
    {
        $config['upload_path']   = './assets/uploads/';
        $config['allowed_types'] = 'gif|jpg|png|jpeg|mp4|avi|mov|wmv';
        $config['max_size']      = 100000; // 100MB
        $config['max_width']     = 10240;  // 10,240px
        $config['max_height']    = 7680;   // 7,680px

        $this->load->library('upload', $config);

        if (! $this->upload->do_upload('userfile')) {
            $error = $this->upload->display_errors();
            $this->session->set_flashdata('error_upload', $error);
        } else {
            $data     = $this->upload->data();
            $file_url = base_url('assets/uploads/' . $data['file_name']);
            $success  = $this->lang->line('admin_file_uploaded');
            $this->session->set_flashdata('success_upload', $success);
            $this->session->set_flashdata('file_url', $file_url);
        }

        redirect('admin/ajustes#tools');
        //redirect('admin/ajustes');

    }

    /**
     * Upload extenal page
     */
    public function uploadpage()
    {

        $this->actualizar_ultima_actividad();

        $data['title']     = $this->lang->line('Upload');
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/uploadpage', $data);
        $this->load->view('templates/footer');

    }

    /**
     * Uploads a zip.
     */
    public function upload_zip()
    {
        // Configurar la ruta base de carga y obtener la página seleccionada
        $upload_path_base = './external/';
        $page             = $this->input->post('page');
        $upload_path      = $upload_path_base . $page . '/';

        // Verificar los límites de tamaño de archivo
        $upload_max_filesize = ini_get('upload_max_filesize');
        $post_max_size       = ini_get('post_max_size');
        $max_size            = min($this->convertToBytes($upload_max_filesize), $this->convertToBytes($post_max_size));

        // Verificar si el archivo se está subiendo
        if (empty($_FILES) || ! isset($_FILES['zipFile'])) {
            if ($_SERVER['CONTENT_LENGTH'] > $max_size) {
                $error = 'The uploaded file exceeds the maximum size allowed by the server configuration. ' .
                    'upload_max_filesize: ' . $upload_max_filesize . ', post_max_size: ' . $post_max_size . '.';
            } else {
                $error = 'No file uploaded or wrong file input name.';
            }
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
            return;
        }

        $zipFile = $_FILES['zipFile'];

        // Verificar el tamaño del archivo
        if ($zipFile['size'] > $max_size) {
            $error = 'The uploaded file exceeds the maximum size allowed by the server configuration. ' .
                'upload_max_filesize: ' . $upload_max_filesize . ', post_max_size: ' . $post_max_size . '.';
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
            return;
        }

        // Verificar si se subió un archivo ZIP
        if ($zipFile['error'] !== UPLOAD_ERR_OK) {
            $error = 'Failed to upload ZIP file.';
            switch ($zipFile['error']) {
                case UPLOAD_ERR_INI_SIZE:
                case UPLOAD_ERR_FORM_SIZE:
                    $error = 'The uploaded file exceeds the maximum size allowed.';
                    break;
                case UPLOAD_ERR_PARTIAL:
                    $error = 'The uploaded file was only partially uploaded.';
                    break;
                case UPLOAD_ERR_NO_FILE:
                    $error = 'No file was uploaded.';
                    break;
                case UPLOAD_ERR_NO_TMP_DIR:
                    $error = 'Missing a temporary folder.';
                    break;
                case UPLOAD_ERR_CANT_WRITE:
                    $error = 'Failed to write file to disk.';
                    break;
                case UPLOAD_ERR_EXTENSION:
                    $error = 'File upload stopped by extension.';
                    break;
                default:
                    $error = 'Unknown upload error.';
                    break;
            }
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
            return;
        }

        // Verificar la extensión del archivo
        $fileExt = pathinfo($zipFile['name'], PATHINFO_EXTENSION);
        if (strtolower($fileExt) !== 'zip') {
            $error = 'Only ZIP files are allowed.';
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
            return;
        }

        // Eliminar cualquier contenido previo en el directorio
        if (file_exists($upload_path)) {
            $this->deleteDirectory($upload_path);
        }

        // Crear el directorio de carga si no existe
        if (! file_exists($upload_path)) {
            if (! mkdir($upload_path, 0755, true)) {
                $error = 'Failed to create directory for page.';
                $this->session->set_flashdata('error', $error);
                echo json_encode(['status' => 'error', 'message' => $error]);
                return;
            }
        }

        // Crear un directorio temporal para descomprimir el archivo
        $tempDir = tempnam(sys_get_temp_dir(), 'zip');
        unlink($tempDir);
        mkdir($tempDir);

        // Mover el archivo ZIP al directorio temporal
        $zipPath = $tempDir . '/' . $zipFile['name'];
        if (! move_uploaded_file($zipFile['tmp_name'], $zipPath)) {
            $error = 'Failed to move uploaded ZIP file.';
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
            return;
        }

        // Descomprimir el archivo ZIP
        $zip = new ZipArchive;
        if ($zip->open($zipPath) === true) {
            $zip->extractTo($upload_path);
            $zip->close();
            unlink($zipPath); // Eliminar el archivo ZIP después de la extracción

            $success = 'ZIP file uploaded and extracted successfully.';
            $this->session->set_flashdata('success', $success);
            echo json_encode(['status' => 'success', 'message' => $success]);
        } else {
            $error = 'Failed to extract ZIP file.';
            $this->session->set_flashdata('error', $error);
            echo json_encode(['status' => 'error', 'message' => $error]);
        }
    }

    /**
     * { convertToBytes }
     *
     * @param      int   $size_str  The size string
     *
     * @return     int   ( description_of_the_return_value )
     */
    private function convertToBytes($size_str)
    {
        switch (substr($size_str, -1)) {
            case 'M':case 'm':return (int) $size_str * 1048576;
            case 'K':case 'k':return (int) $size_str * 1024;
            case 'G':case 'g':return (int) $size_str * 1073741824;
            default:return (int) $size_str;
        }
    }

    /**
     * { deleteDirectory}
     *
     * @param      <type>  $dir    The dir
     *
     * @return     bool    ( description_of_the_return_value )
     */
    private function deleteDirectory($dir)
    {
        if (! file_exists($dir)) {
            return true;
        }

        if (! is_dir($dir)) {
            return unlink($dir);
        }

        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') {
                continue;
            }

            if (! $this->deleteDirectory($dir . DIRECTORY_SEPARATOR . $item)) {
                return false;
            }
        }

        return rmdir($dir);
    }

    /**
     * { cambiar_plantilla }
     */
    public function cambiar_plantilla()
    {
        $this->load->model('Config_model');
        $plantilla = $this->input->post('plantilla'); // Asegúrate de usar el nombre correcto del campo
        $this->Config_model->set_config_value('tipo_plantilla', $plantilla);
        $this->session->set_flashdata('alert-success', $this->lang->line('template_type_changed_success'));
        redirect('admin/ajustes');
    }

    /**
     * Change font
     */
    public function change_font()
    {
        $this->load->model('Config_model');
        $font = $this->input->post('font');
        $this->Config_model->set_config_value('font_family', $font);
        $this->Config_model->set_config_value('custom_font', ''); // Limpiar fuente personalizada
        $this->session->set_flashdata('alert-success', $this->lang->line('google_font_changed_success'));
        redirect('admin/ajustes');
    }

    /**
     * Uploads a custom font.
     */
    public function upload_custom_font()
    {
        $this->load->model('Config_model');
        $config['upload_path']   = './assets/fonts/';
        $config['allowed_types'] = 'ttf|otf|woff|woff2';
        $config['max_size']      = 2048;

        $this->load->library('upload', $config);

        if (! $this->upload->do_upload('custom_font')) {
            $error = $this->upload->display_errors();
            log_message('error', 'Error uploading font: ' . $error);
            $this->session->set_flashdata('alert-danger', $error);
        } else {
            $upload_data = $this->upload->data();
            $font_path   = 'assets/fonts/' . $upload_data['file_name']; // Note the change here
            $this->Config_model->set_config_value('custom_font', $font_path);
            $this->Config_model->set_config_value('font_family', ''); // Limpiar fuente de Google Fonts
            $this->session->set_flashdata('alert-success', $this->lang->line('custom_font_uploaded_success'));
        }

        redirect('admin/ajustes');
    }

    public function activar_registro()
    {

        $data['title']     = "Activar/Desactivar Registro de Usuarios";
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Obtener el estado actual de la configuración de activación del registro de usuarios
        $data['activar'] = $this->Config_model->get_config_value('activar_registro');

        $this->form_validation->set_rules('activar', 'activar', 'in_list[0,1]');

        if (! $this->form_validation->run()) {
            $this->load->view('templates/header', $data);
            $this->load->view('templates/sidebar', $data);
            $this->load->view('templates/topbar', $data);
            $this->load->view('admin/activar_registro', $data);
            $this->load->view('templates/footer');
        } else {
            $activar = $this->input->post('activar');

            // Actualizar la configuración del registro de usuarios
            $this->Config_model->update_config('activar_registro', $activar);

            $this->session->set_flashdata('alert-success', $this->lang->line('user_activation_modified'));
            redirect('admin/ajustes');
        }
    }

    public function activar_menu_lateral()
    {

        $data['title']     = "Activar Menu Lateral para Usuarios";
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Obtener el estado actual de la configuración de activación del menú lateral
        $data['activar'] = $this->Config_model->get_config_value('activar_menu_lateral');

        $this->form_validation->set_rules('activar', 'activar', 'in_list[0,1]');

        if (! $this->form_validation->run()) {
            $this->load->view('templates/header', $data);
            $this->load->view('templates/sidebar', $data);
            $this->load->view('templates/topbar', $data);
            $this->load->view('admin/activar_registro', $data);
            $this->load->view('templates/footer');
        } else {
            $activar = $this->input->post('activar');

            // Actualizar la configuración del menú lateral
            $this->Config_model->update_config('activar_menu_lateral', $activar);

            $this->session->set_flashdata('alert-success', $this->lang->line('sidebar_menu_activation_modified'));
            redirect('admin/ajustes');
        }
    }

    public function activar_default_activation()
    {

        $data['title']     = "Activar default_activation";
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Obtener el estado actual de la configuración de activación por defecto
        $data['activar'] = $this->Config_model->get_config_value('default_activation');

        $this->form_validation->set_rules('activar', 'activar', 'in_list[0,1]');

        if (! $this->form_validation->run()) {
            $this->load->view('templates/header', $data);
            $this->load->view('templates/sidebar', $data);
            $this->load->view('templates/topbar', $data);
            $this->load->view('admin/activar_registro', $data);
            $this->load->view('templates/footer');
        } else {
            $activar = $this->input->post('activar');

            // Actualizar la configuración de activación por defecto
            $this->Config_model->update_config('default_activation', $activar);

            $this->session->set_flashdata('alert-success', $this->lang->line('default_activation_changed'));
            redirect('admin/ajustes');
        }
    }

    public function update_styles_login_appearance()
    {
        if ($_POST) {
            $config_updates = [
                'background_container' => $this->input->post('background_container'),
                'text_header'          => $this->input->post('text_header'),

                'button_color'         => $this->input->post('button_color'),
                'email_input'          => $this->input->post('email_input'),
                'password_input'       => $this->input->post('password_input'),
                'checkbox_text'        => $this->input->post('checkbox_text'),
                'posicion'             => $this->input->post('posicion'),
                'forgot_create'        => $this->input->post('forgot_create'),

                'css_placeholder'      => $this->input->post('css_placeholder'),

                'css_focus'            => $this->input->post('css_focus'),
                'css_general'          => $this->input->post('css_general'),

            ];

            foreach ($config_updates as $key => $value) {
                $this->Config_model->update_config_value($key, $value);
            }

            redirect('admin/ajustes#configuracionlogin');
            //redirect('admin/ajustes');
        }
    }

    public function update_styles_text()
    {
        if ($_POST) {
            $config_updates = [

                'texto_login'                => $this->input->post('texto_login'),
                'texto_email_placeholder'    => $this->input->post('texto_email_placeholder'),
                'texto_password_placeholder' => $this->input->post('texto_password_placeholder'),
                'texto_terminos'             => $this->input->post('texto_terminos'),
                'texto_boton'                => $this->input->post('texto_boton'),
                'texto_forgot'               => $this->input->post('texto_forgot'),
                'texto_crear_cuenta'         => $this->input->post('texto_crear_cuenta'),

                'contenido_sidebar'          => $this->input->post('contenido_sidebar'),
                'contenido_terminos'         => $this->input->post('contenido_terminos'),
                'source_image'               => $this->input->post('source_image'),

            ];

            foreach ($config_updates as $key => $value) {
                $this->Config_model->update_config_value($key, $value);
            }

            redirect('admin/ajustes#textos');
            //redirect('admin/ajustes');

        }
    }

    public function update_styles()
    {
        if ($_POST) {
            $config_updates = [
                'background_container'       => $this->input->post('background_container'),
                'text_header'                => $this->input->post('text_header'),

                'button_color'               => $this->input->post('button_color'),
                'email_input'                => $this->input->post('email_input'),
                'password_input'             => $this->input->post('password_input'),
                'checkbox_text'              => $this->input->post('checkbox_text'),
                'posicion'                   => $this->input->post('posicion'),
                'forgot_create'              => $this->input->post('forgot_create'),
                'animation_effect'           => $this->input->post('animation_effect'),
                'texto_login'                => $this->input->post('texto_login'),
                'texto_email_placeholder'    => $this->input->post('texto_email_placeholder'),
                'texto_password_placeholder' => $this->input->post('texto_password_placeholder'),
                'texto_terminos'             => $this->input->post('texto_terminos'),
                'texto_boton'                => $this->input->post('texto_boton'),
                'texto_forgot'               => $this->input->post('texto_forgot'),
                'texto_crear_cuenta'         => $this->input->post('texto_crear_cuenta'),

                'css_focus'                  => $this->input->post('css_focus'),
                'css_general'                => $this->input->post('css_general'),
                'contenido_sidebar'          => $this->input->post('contenido_sidebar'),
                'contenido_terminos'         => $this->input->post('contenido_terminos'),
                'source_image'               => $this->input->post('source_image'),
                'footer'                     => $this->input->post('footer'),
            ];

            foreach ($config_updates as $key => $value) {
                $this->Config_model->update_config_value($key, $value);
            }

            redirect('admin/ajustes');
        }
    }

    public function update_styles2()
    {
        // Verificar si es una solicitud POST válida y que contiene el dato 'key'
        if ($this->input->method() == 'post' && $this->input->post('key') !== null) {
            // Obtener los datos del POST
            $key   = $this->input->post('key');
            $value = $this->input->post('value');

            // Actualizar el valor en la base de datos utilizando tu modelo
            $this->Config_model->update_config_value($key, $value);

            // Devolver una respuesta JSON para indicar éxito
            echo json_encode(['success' => true]);
        } else {
            // Devolver un mensaje de error si no es una solicitud POST válida o falta 'key'
            echo json_encode(['error' => $this->lang->line('admin_no_valid_data')]);
        }
    }

    public function restore_default($key)
    {
        $default_values = [

            'activar_registro'           => '1',
            'activar_menu_lateral'       => '1',
            'default_activation'         => '1',
            'tipo_plantilla'             => 'Normal',
            'font_family'                => '',

            'gradiente'                  => 'background:black;',
            'gradiente2'                 => 'background:blue;',
            'gradiente3'                 => 'background:black;',
            'page_background'            => 'background:white;',
            'background_container'       => 'background-color: white;padding:2rem;',
            'text_header'                => 'color: #333333;',

            'button_color'               => 'background-color: #007bff; color: #ffffff;border-radius:20px',
            'email_input'                => 'border-radius:20px',
            'password_input'             => 'border-radius:20px',
            'checkbox_text'              => 'color:gray;text-align: center;font-size: 12px;',
            'posicion'                   => 'display: flex; justify-content: center; align-items: center; height: 100vh;',
            'forgot_create'              => 'color:#0066CC;text-align: center;',
            'animation_effect'           => '',
            'texto_login'                => $this->lang->line('texto_login'),
            'texto_email_placeholder'    => $this->lang->line('texto_email_placeholder'),
            'texto_password_placeholder' => $this->lang->line('texto_password_placeholder'),
            'texto_terminos'             => $this->lang->line('texto_terminos'),
            'texto_boton'                => $this->lang->line('texto_boton'),
            'texto_forgot'               => $this->lang->line('texto_forgot'),
            'texto_crear_cuenta'         => $this->lang->line('texto_crear_cuenta'),
            'contenido_sidebar'          => '<br>
         <div style="color:white" >

         <span id="openSidebarBtn">&#9776; Menu</span>

         <div class="container mt-5" style="color:white">

         <div class="container-fluid" style="height: 100%; display: flex; flex-direction: column; justify-content: center; align-items: center;">
         <div class="row w-100">
         <div class="col-12 text-center">
         <h1 class="display-4">Título del Producto</h1>
         <p class="lead">Descripción breve del producto que puede ocupar una o dos líneas. Puede incluir características clave y beneficios.</p>
         </div>
         </div>

         </div>
         </div>
         </div>',
            'contenido_terminos'         => $this->lang->line('terms_conditions'),
            'css_placeholder'            => '',
            'css_focus'                  => '',
            'css_general'                => '',
            'footer'                     => '<footer class="sticky-footer bg-white">
         <div class="container my-auto">
         <div class="copyright text-center my-auto">
         <span id="copyright">Copyright &copy; emo ' . date("Y") . '</span>
         </div>
         </div>
         </footer>',

        ];

        if (array_key_exists($key, $default_values)) {
            $new_value = $default_values[$key];
            $this->Config_model->update_config_value($key, $new_value);

            // Return the new value as a JSON response
            echo json_encode(['value' => $new_value]);
        } else {
            // Return an error message if the key is not found
            echo json_encode(['error' => 'Invalid key']);
        }

        // redirect('admin/ajustes');
    }

    //restaura todos los valores por lo de por defecto de una vez:

    public function restore_all_defaults()
    {
        $default_values = [
            'activar_registro'           => '1',
            'activar_menu_lateral'       => '1',
            'default_activation'         => '1',

            'tipo_plantilla'             => "normal",
            'font_family'                => "",
            'gradiente'                  => 'background:black;',
            'gradiente2'                 => 'background:blue;',
            'gradiente3'                 => 'background:black;',
            'page_background'            => 'background:white;',
            'background_container'       => 'background-color: white;padding:2rem;',
            'text_header'                => 'color: #333333;',

            'button_color'               => 'background-color: #007bff; color: #ffffff;border-radius:20px;',
            'email_input'                => 'border-radius:20px',
            'password_input'             => 'border-radius:20px',
            'checkbox_text'              => 'color:gray;text-align: center;font-size: 12px;',
            'posicion'                   => 'display: flex; justify-content: center; align-items: center; height: 100vh;',
            'forgot_create'              => 'color:#0066CC;text-align: center;',
            'animation_effect'           => '',
            'texto_login'                => $this->lang->line('texto_login'),
            'texto_email_placeholder'    => $this->lang->line('texto_email_placeholder'),
            'texto_password_placeholder' => $this->lang->line('texto_password_placeholder'), // Asegúrate de definir esto en tu archivo de idioma
            'texto_terminos' => $this->lang->line('texto_terminos'),
            'texto_boton'                => $this->lang->line('texto_boton'),
            'texto_forgot'               => $this->lang->line('texto_forgot'),
            'texto_crear_cuenta'         => $this->lang->line('texto_crear_cuenta'),
            'contenido_sidebar'          => '<br>
            <div style="color:white" >

            <span id="openSidebarBtn">&#9776; Menu</span>

            <div class="container mt-5" style="color:white">

            <div class="container-fluid" style="height: 100%; display: flex; flex-direction: column; justify-content: center; align-items: center;">
            <div class="row w-100">
            <div class="col-12 text-center">
            <h1 class="display-4">Título del Producto</h1>
            <p class="lead">Descripción breve del producto que puede ocupar una o dos líneas. Puede incluir características clave y beneficios.</p>
            </div>
            </div>

            </div>
            </div>
            </div>',
            'contenido_terminos'         => $this->lang->line('terms_conditions'),
            'css_placeholder'            => '',
            'css_focus'                  => '',
            'css_general'                => '',
            'source_image'               => '',
            'footer'                     => '<footer class="sticky-footer bg-white">
            <div class="container my-auto">
            <div class="copyright text-center my-auto">
            <span id="copyright">Copyright &copy; emo ' . date("Y") . '</span>
            </div>
            </div>
            </footer>',
        ];

        // Iterar sobre el array de valores por defecto y actualizar cada uno en la base de datos
        foreach ($default_values as $key => $new_value) {
            $this->Config_model->update_config_value($key, $new_value);
        }

        // Redireccionar después de actualizar los valores
        redirect('admin/ajustes#textos');
        //redirect('admin/ajustes');
    }

    //restaura solo los textos al lenjuaje por defecto de la app
    public function restore_text_defaults()
    {
        // Cargar el archivo de configuración
        $config_file = APPPATH . 'config/config.php';
        $config      = include $config_file;

        // Obtener el idioma configurado
        $language = $config['language'];

        // Cargar el archivo de idioma correspondiente
        $this->lang->load('aplicacion_lang', $language);

        // Valores por defecto a restaurar
        $default_values = [
            'texto_login'                => $this->lang->line('texto_login'),
            'texto_email_placeholder'    => $this->lang->line('texto_email_placeholder'),
            'texto_password_placeholder' => $this->lang->line('texto_password_placeholder'),
            'texto_terminos'             => $this->lang->line('texto_terminos'),
            'texto_boton'                => $this->lang->line('texto_boton'),
            'texto_forgot'               => $this->lang->line('texto_forgot'),
            'texto_crear_cuenta'         => $this->lang->line('texto_crear_cuenta'),
            'contenido_terminos'         => $this->lang->line('terms_conditions'),
        ];

        // Iterar sobre el array de valores por defecto y actualizar cada uno en la base de datos
        foreach ($default_values as $key => $new_value) {
            $this->Config_model->update_config_value($key, $new_value);
        }

        // Redireccionar después de actualizar los valores
        redirect('admin/ajustes#textos');
        //redirect('admin/ajustes');
    }

    // Export data to JSON
    public function export_config()
    {
/*		header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: GET, POST, OPTIONS');
header('Access-Control-Allow-Headers: Content-Type');*/

        $query = $this->db->get('config');
        $data  = $query->result_array();

        $json_data = json_encode($data);

        $this->output
            ->set_content_type('application/json')
            ->set_output($json_data);
    }

    public function import_config()
    {

        $json_data = file_get_contents('php://input');
        $data      = json_decode($json_data, true);

        if (json_last_error() === JSON_ERROR_NONE && ! empty($data)) {
            // Validar la estructura de los datos (ejemplo básico)
            foreach ($data as $item) {
                if (! isset($item['key']) || ! isset($item['value'])) {
                    echo json_encode(['success' => false, 'message' => $this->lang->line('invalid_data_structure')]);
                    return;
                }
            }

            // Filtrado/Escapado de Datos si es necesario
            // Aquí se podría aplicar escapado adicional según sea necesario

            // Empezar la transacción
            $this->db->trans_start();

            // Insertar o actualizar cada clave-valor
            foreach ($data as $item) {
                // Verificar si la clave ya existe
                $this->db->where('key', $item['key']);
                $query = $this->db->get('config');

                if ($query->num_rows() > 0) {
                    // Si la clave existe, actualizar el valor
                    $this->db->where('key', $item['key']);
                    $this->db->update('config', ['value' => $item['value']]);
                } else {
                    // Si la clave no existe, insertar nuevo registro
                    $this->db->insert('config', $item);
                }
            }

            // Completar la transacción
            $this->db->trans_complete();

            if ($this->db->trans_status() === false) {
                echo json_encode(['success' => false, 'message' => $this->lang->line('failed_to_import_data')]);
            } else {
                echo json_encode(['success' => true, 'message' => $this->lang->line('data_imported_ok')]);
            }
        } else {
            echo json_encode(['success' => false, 'message' => $this->lang->line('invalid_json_data')]);
        }
    }

    public function get_config_json()
    {
        $config = $this->db->get('config')->result_array(); // Consulta directa a la base de datos

        header('Content-Type: application/json');
        echo json_encode($config);
    }

    public function archive_design()
    {
        $titulo      = $this->input->post('titulo');
        $descripcion = $this->input->post('descripcion');
        $json        = $this->input->post('json');

        $imagen = null;
        if (! empty($_FILES['imagen']['name'])) {
            $imagen = file_get_contents($_FILES['imagen']['tmp_name']);
        }

        $data = [
            'titulo'      => $titulo,
            'descripcion' => $descripcion,
            'imagen'      => $imagen,
            'json'        => $json,
        ];

        if ($this->Config_model->insert_archive($data)) {
            echo json_encode(['success' => true]);
        } else {
            echo json_encode(['success' => false, 'message' => $this->lang->line('error_archiving_design')]);
        }
    }

    public function get_archived_images()
    {
        $this->load->model('Config_model');
        $result = $this->Config_model->get_archived_images();

        if ($result) {
            foreach ($result as &$row) {
                $row['imagen'] = base64_encode($row['imagen']); // Convertir imagen a base64
            }
            echo json_encode($result);
        } else {
            echo json_encode([]);
        }
    }

    public function delete_image()
    {
        $id = $this->input->post('id');
        // Lógica para eliminar la imagen de la base de datos
        $this->db->where('id', $id);
        if ($this->db->delete('archivo')) {
            echo json_encode(['success' => true]);
        } else {
            echo json_encode(['success' => false, 'message' => $this->lang->line('error_deleting_image')]);
        }
    }

    public function editar_image()
    {
        // Obtener los datos del formulario
        $id          = $this->input->post('id');
        $titulo      = $this->input->post('titulo');
        $descripcion = $this->input->post('descripcion');
        $json        = $this->input->post('json'); // Asumiendo que 'json' es un campo de texto normal en la tabla

        // Procesar la imagen si se ha subido una nueva
        if (isset($_FILES['imagen']) && $_FILES['imagen']['size'] > 0) {
            $imagen = file_get_contents($_FILES['imagen']['tmp_name']);
        } else {
            // Si no se sube una nueva imagen, mantén la existente
            $this->db->select('imagen');
            $this->db->where('id', $id);
            $query  = $this->db->get('archivo');
            $row    = $query->row();
            $imagen = $row->imagen;
        }

        // Datos a actualizar
        $update_data = [
            'titulo'      => $titulo,
            'descripcion' => $descripcion,
            'imagen'      => $imagen,
            'json'        => $json,
        ];

        // Actualizar el diseño en la base de datos
        $this->db->where('id', $id);
        $result = $this->db->update('archivo', $update_data);

        if ($result) {
            echo json_encode(['success' => true]);
        } else {
            echo json_encode(['success' => false, 'message' => $this->lang->line('problem_updating_design')]);
        }
    }

    public function restore_config()
    {
        $id            = $this->input->post('id');
        $archived_data = $this->Config_model->get_archive_by_id($id);

        if ($archived_data) {
            $config_data = json_decode($archived_data['json'], true);

            // Verificar si JSON es válido
            if (json_last_error() !== JSON_ERROR_NONE) {
                echo json_encode(['success' => false, 'message' => $this->lang->line('archived_json_invalid')]);
                return;
            }

            // Verificar estructura del JSON
            if (! is_array($config_data) || empty($config_data)) {
                echo json_encode(['success' => false, 'message' => $this->lang->line('invalid_json_structure_archived')]);
                return;
            }

            // Comenzar transacción
            $this->db->trans_begin();

            try {
                foreach ($config_data as $config_item) {
                    // Verificar si ya existe un registro con la misma key en la tabla config
                    $existing_config = $this->Config_model->get_config_value($config_item['key']);

                    if ($existing_config) {
                        // Si existe, actualizar el registro
                        $this->db->where('key', $config_item['key']);
                        $this->db->update('config', $config_item);
                    } else {
                        // Si no existe, insertar un nuevo registro
                        $this->db->insert('config', $config_item);
                    }
                }

                // Verificar estado de la transacción
                if ($this->db->trans_status() === false) {
                    // Rollback transaction
                    $this->db->trans_rollback();
                    echo json_encode(['success' => false, 'message' => $this->lang->line('error_resetting_configuration')]);
                } else {
                    // Commit transaction
                    $this->db->trans_commit();
                    echo json_encode(['success' => true]);
                }
            } catch (Exception $e) {
                // Capturar excepciones
                $this->db->trans_rollback();
                echo json_encode(['success' => false, 'message' => $this->lang->line('error_processing_json_data') . ': ' . $e->getMessage()]);
            }
        } else {
            echo json_encode(['success' => false, 'message' => $this->lang->line('role_id_required')]);
        }
    }

    public function role()
    {
        $this->actualizar_ultima_actividad();
        $data['title']     = $this->lang->line('Role');
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Obtener todos los roles para mostrar en la vista
        $data['roles'] = $this->User_model->getRoles();

        // Validación del formulario
        $this->form_validation->set_rules('role', 'Role', 'required');
        $this->form_validation->set_rules('redirection', 'Redirection', 'valid_url');

        if ($this->form_validation->run()) {
            // Si el formulario se envía correctamente
            $roleData = [
                'role'        => $this->input->post('role'),
                'redirection' => $this->input->post('redirection') ?: null, // Asegurar que el valor sea null si está vacío
            ];

            $this->User_model->addRole($roleData);

            $this->session->set_flashdata('alert-success', $this->lang->line('Role_has_been_added'));
            redirect('admin/role');
        }

        // Cargar la vista con los datos necesarios
        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/role', $data);
        $this->load->view('templates/footer');
    }

    public function editRole()
    {
        $id          = $this->input->post('id');
        $role        = $this->input->post('role');
        $redirection = $this->input->post('redirection');

        if (empty($id) || empty($role)) {
            $this->session->set_flashdata('alert-danger', $this->lang->line('role_id_or_name_missing'));
            redirect('admin/role');
            return;
        }

        $this->form_validation->set_rules('role', 'Role Name', 'required');

        if ($this->form_validation->run()) {
            $data = [
                'role'        => $role,
                'redirection' => $redirection,
            ];

            if ($this->User_model->editRole($id, $data)) {
                $this->session->set_flashdata('alert-success', $this->lang->line('Role_has_been_edited'));
            } else {
                $this->session->set_flashdata('alert-danger', $this->lang->line('error_updating_role'));
            }

            redirect('admin/role');
        } else {
            $this->session->set_flashdata('alert-danger', validation_errors());
            redirect('admin/role');
        }
    }

    public function deleteRole($id = null)
    {
        // id = null
        if ($id <= 2) {
            //
            if ($id > 0) {
                $this->session->set_flashdata('alert-danger', $this->lang->line('This_role_cannot_be_delete'));
            }
            redirect('admin/role');
        }

        $this->User_model->deleteRole($id);
        $this->session->set_flashdata('alert-success', $this->lang->line('Role_has_been_deleted'));
        redirect('admin/role');
    }

    public function roleAccess($role_id = null)
    {
        if (is_null($role_id)) {
            redirect('auth/blocked');
            die();
        }

        $data['title']     = $this->lang->line('Role_Access');
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $this->load->model('Menu_model');
        $data['role']  = $this->User_model->getRole($role_id);
        $data['menus'] = $this->Menu_model->getMenus();

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/role-access', $data);
        $this->load->view('templates/footer');
    }

    public function changeAccess()
    {
        if (empty($this->input->post())) {
            redirect('auth/blocked');
        }

        $menuId = $this->input->post('menuId');
        $roleId = $this->input->post('roleId');

        $data = [
            'role_id' => $roleId,
            'menu_id' => $menuId,
        ];

        $result = $this->User_model->getAccess($data);

        if ($result->num_rows() < 1) {
            $this->User_model->addAccess($data);
        } else {
            $this->User_model->deleteAccess($data);
        }

        $this->session->set_flashdata('message', $this->lang->line('Access_Changed'));
    }

    public function getRole()
    {
        $role_id = $this->input->get('role_id');
        if ($role_id) {
            $role = $this->User_model->getRole($role_id);
            echo json_encode($role);
        } else {
            echo json_encode(['error' => $this->lang->line('role_id_required')]);
        }
    }

    public function record_download()
    {
        // Obtener datos JSON del cuerpo de la solicitud
        $postData = json_decode(file_get_contents('php://input'), true);
        log_message('debug', 'Post Data: ' . json_encode($postData));

        $user_id    = $this->user['id'];    // Obtener el user_id desde los datos del usuario
        $user_name  = $this->user['name'];  // Obtener el nombre del usuario
        $user_email = $this->user['email']; // Obtener el email del usuario
        log_message('debug', 'User ID from session: ' . $user_id);

        if (isset($postData['file_name'])) {
            $file_name = $postData['file_name'];
            log_message('debug', 'File Name: ' . $file_name);

            if ($user_id && $file_name) {
                $data = [
                    'user_id'    => $user_id,
                    'user_name'  => $user_name,
                    'user_email' => $user_email,
                    'file_name'  => $file_name,
                ];
                $this->db->insert('downloads', $data);

                // Verificar si el archivo ya existe en la tabla `files`
                $this->db->where('file_name', $file_name);
                $query = $this->db->get('files');
                if ($query->num_rows() > 0) {
                    // Si el archivo existe, actualizar el conteo de descargas
                    $this->db->set('downloads_count', 'downloads_count+1', false);
                    $this->db->where('file_name', $file_name);
                    $this->db->update('files');
                } else {
                    // Si el archivo no existe, insertar un nuevo registro
                    $file_data = [
                        'file_name'       => $file_name,
                        'downloads_count' => 1,
                    ];
                    $this->db->insert('files', $file_data);
                }

                // Obtener el nuevo conteo de descargas
                $this->db->select('downloads_count');
                $this->db->from('files');
                $this->db->where('file_name', $file_name);
                $query  = $this->db->get();
                $result = $query->row();

                echo json_encode(['status' => 'success', 'downloads_count' => $result->downloads_count]);
            } else {
                log_message('error', 'User ID or file name missing');
                echo json_encode(['status' => 'error', 'message' => 'User ID or file name missing']);
            }
        } else {
            log_message('error', 'Invalid data received');
            echo json_encode(['status' => 'error', 'message' => 'Invalid data']);
        }
    }

    public function seo_settings()
    {
                                         // Obtener configuración SEO
        $this->load->model('Seo_model'); // Cargar el modelo SEO
        $data['seo']       = $this->Seo_model->get_seo_settings();
        $data['title']     = 'SEO';
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/seo_view', $data);
        $this->load->view('templates/footer');
    }

    public function update_seo_settings()
    {
        $this->form_validation->set_rules('title', 'Title', 'required');
        $this->form_validation->set_rules('description', 'Description', 'required');
        $this->form_validation->set_rules('keywords', 'Keywords', 'required');
        $this->form_validation->set_rules('author', 'Author', 'required');
        $this->form_validation->set_rules('og_title', 'OG Title', 'required');
        $this->form_validation->set_rules('og_description', 'OG Description', 'required');
        $this->form_validation->set_rules('og_image', 'OG Image', 'required');

        if ($this->form_validation->run() == false) {
            $this->session->set_flashdata('error', validation_errors());
            redirect('admin/seo_settings');
        } else {
            $data = [
                'title'          => $this->input->post('title'),
                'description'    => $this->input->post('description'),
                'keywords'       => $this->input->post('keywords'),
                'author'         => $this->input->post('author'),
                'og_title'       => $this->input->post('og_title'),
                'og_description' => $this->input->post('og_description'),
                'og_image'       => $this->input->post('og_image'),
            ];
            if ($this->update_seo_settings_in_db($data)) {
                $this->session->set_flashdata('success', 'SEO settings updated successfully.');
            } else {
                $this->session->set_flashdata('error', 'Failed to update SEO settings.');
            }
            redirect('admin/seo_settings');
        }
    }

    private function get_seo_settings()
    {
        $query = $this->db->get('seo');
        if ($query->num_rows() > 0) {
            return $query->row_array();
        } else {
            return [
                'title'          => 'Default Title',
                'description'    => 'Default description for the site.',
                'keywords'       => 'default, keywords, here',
                'author'         => 'Default Author',
                'og_title'       => 'Default OG Title',
                'og_description' => 'Default OG description for the site.',
                'og_image'       => base_url('path/to/default/image.jpg'),
            ];
        }
    }

    private function update_seo_settings_in_db($data)
    {
        if ($this->db->count_all('seo') == 0) {
            return $this->db->insert('seo', $data);
        } else {
            return $this->db->update('seo', $data);
        }
    }

    public function toggle_maintenance()
    {
        $maintenance_mode = $this->input->post('maintenance_mode') === 'on' ? true : false;

        $config_path = APPPATH . 'config/config.php';
        $config_file = read_file($config_path);

        if ($maintenance_mode) {
            $new_config = str_replace('$config[\'maintenance_mode\'] = FALSE;', '$config[\'maintenance_mode\'] = TRUE;', $config_file);
        } else {
            $new_config = str_replace('$config[\'maintenance_mode\'] = TRUE;', '$config[\'maintenance_mode\'] = FALSE;', $config_file);
        }

        if (write_file($config_path, $new_config)) {
            $this->session->set_flashdata('message', 'Maintenance mode ' . ($maintenance_mode ? 'enabled' : 'disabled') . '.');
        } else {
            $this->session->set_flashdata('message', 'Failed to update maintenance mode.');
        }

        redirect('admin');
    }

    public function edit_pwa()
    {
        $manifest_path       = FCPATH . 'assets/manifest.json';
        $service_worker_path = FCPATH . 'assets/service-worker.js';

        $data['manifest_content']       = file_get_contents($manifest_path);
        $data['service_worker_content'] = file_get_contents($service_worker_path);

        // Ruta del archivo header
        $file_path = APPPATH . 'views/templates/auth_header.php';

        // Lee el contenido actual del archivo
        $file_content = file_get_contents($file_path);

        // Extrae el contenido del script entre los marcadores
        $pattern = '/\/\/PWA_SCRIPT_START\/\/([\s\S]*?)\/\/PWA_SCRIPT_END\/\//';
        preg_match($pattern, $file_content, $matches);
        $script_content = isset($matches[1]) ? trim($matches[1]) : '';

        // Cargar la vista y pasar el contenido del script
        $data['script_content'] = $script_content;

        $data['title']     = 'PWA';
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Añadir mensaje de éxito o error
        $data['message'] = $this->session->flashdata('message');

        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/edit_pwa', $data);
        $this->load->view('templates/footer');
    }

    public function save_pwa()
    {
        $manifest_content       = $this->input->post('manifest_content');
        $service_worker_content = $this->input->post('service_worker_content');

        $manifest_path       = FCPATH . 'assets/manifest.json';
        $service_worker_path = FCPATH . 'assets/service-worker.js';

        $saved = file_put_contents($manifest_path, $manifest_content) !== false && file_put_contents($service_worker_path, $service_worker_content) !== false;

        if ($saved) {
            $this->session->set_flashdata('message', 'success');
        } else {
            $this->session->set_flashdata('message', 'error');
        }

        redirect('admin/edit_pwa');
    }

    public function save_pwa_script()
    {
        // Obtén el contenido del script desde el formulario
        $pwa_script_content = $this->input->post('pwa_script_content');

        // Ruta del archivo header
        $file_path = APPPATH . 'views/templates/auth_header.php';

        // Lee el contenido actual del archivo
        $file_content = file_get_contents($file_path);

        // Define el patrón para los marcadores
        $pattern = '/\/\/PWA_SCRIPT_START\/\/([\s\S]*?)\/\/PWA_SCRIPT_END\/\//s';

        // Define el nuevo contenido del script
        $new_script = "//PWA_SCRIPT_START//\n" . $pwa_script_content . "\n//PWA_SCRIPT_END//";

        // Reemplaza el contenido entre los marcadores
        if (preg_match($pattern, $file_content)) {
            $new_file_content = preg_replace($pattern, $new_script, $file_content);
        } else {
            $new_file_content = str_replace('</head>', $new_script . "\n</head>", $file_content);
        }

        // Escribe el contenido actualizado de nuevo en el archivo
        file_put_contents($file_path, $new_file_content);

        // Redirige de nuevo a la página con un mensaje de éxito
        $this->session->set_flashdata('message', 'success');
        redirect('admin/edit_pwa');
    }

    public function install_update()
    {
        if ($this->input->server('REQUEST_METHOD') === 'POST') {
            $timestamp = time();
            $zipUrl    = 'https://loginbasicoemo.com/updates/latest_version.zip?ts=' . $timestamp;

            $updateDir  = sys_get_temp_dir(); // Directorio temporal del sistema
            $zipFile    = $updateDir . '/latest_version.zip';
            $extractDir = $updateDir . '/latest_version';

            // Crear el directorio de actualización si no existe
            if (! file_exists($updateDir)) {
                mkdir($updateDir, 0777, true);
            }

            // Descargar el archivo ZIP
            if (@file_put_contents($zipFile, fopen($zipUrl, 'r')) === false) {
                throw new Exception('Failed to download ZIP file from ' . $zipUrl);
            }

            // Descomprimir el archivo ZIP
            $zip = new ZipArchive;
            if ($zip->open($zipFile) === true) {
                $zip->extractTo($extractDir);
                $zip->close();

                // Copiar los archivos necesarios
                $this->recurse_copy($extractDir . '/application/config', FCPATH . 'application/config');
                $this->recurse_copy($extractDir . '/application/controllers', FCPATH . 'application/controllers');
                $this->recurse_copy($extractDir . '/application/models', FCPATH . 'application/models');
                $this->recurse_copy($extractDir . '/application/language', FCPATH . 'application/language');
                $this->recurse_copy($extractDir . '/application/libraries', FCPATH . 'application/libraries');
                $this->recurse_copy($extractDir . '/application/views', FCPATH . 'application/views');

                // Eliminar el archivo ZIP y el directorio temporal
                unlink($zipFile);
                $this->delete_directory($extractDir);

                echo 'Installation completed successfully';
            } else {
                echo 'Failed to unzip file';
            }
        } else {
            show_404();
        }
    }

    private function recurse_copy($src, $dst)
    {
        $dir = opendir($src);
        @mkdir($dst);
        while (($file = readdir($dir)) !== false) {
            if ($file != '.' && $file != '..') {
                if (is_dir($src . '/' . $file)) {
                    $this->recurse_copy($src . '/' . $file, $dst . '/' . $file);
                } else {
                    copy($src . '/' . $file, $dst . '/' . $file);
                }
            }
        }
        closedir($dir);
    }

    private function delete_directory($dir)
    {
        if (! file_exists($dir)) {
            return true;
        }
        if (! is_dir($dir)) {
            return unlink($dir);
        }
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') {
                continue;
            }
            if (! $this->delete_directory($dir . DIRECTORY_SEPARATOR . $item)) {
                return false;
            }
        }
        return rmdir($dir);
    }

    public function backup_site()
    {

        // Crear un backup de la base de datos
        $prefs = [
            'format'   => 'txt',
            'filename' => 'db_backup.sql',
        ];
        $backup = $this->dbutil->backup($prefs);

        // Agregar el archivo de la base de datos al zip
        $this->zip->add_data('db_backup.sql', $backup);

        // Añadir todos los archivos del sitio al archivo zip
        $path    = FCPATH;
        $exclude = ['tmp', 'cache', 'logs']; // Excluir ciertos directorios si es necesario
        $this->add_files_to_zip($path, '', $exclude);

        // Guardar el archivo zip en un directorio temporal
        $zip_filename = 'site_backup_' . date('Ymd_His') . '.zip';
        $zip_filepath = sys_get_temp_dir() . '/' . $zip_filename;
        $this->zip->archive($zip_filepath);

        // Enviar una respuesta indicando que el backup está listo
        header('Content-Type: application/json');
        echo json_encode(['status' => 'ready', 'url' => site_url('admin/download_backup/' . $zip_filename)]);
        exit;
    }

    public function download_backup($filename)
    {
        // Ruta completa del archivo de backup
        $filepath = sys_get_temp_dir() . '/' . $filename;

        if (file_exists($filepath)) {
            // Forzar la descarga del archivo
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($filepath) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($filepath));
            readfile($filepath);
            exit;
        } else {
            show_404();
        }
    }

    private function add_files_to_zip($path, $relative_path, $exclude = [])
    {
        $dir = opendir($path);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                if (is_dir($path . '/' . $file)) {
                    if (! in_array($file, $exclude)) {
                        $this->add_files_to_zip($path . '/' . $file, $relative_path . $file . '/', $exclude);
                    }
                } else {
                    $this->zip->read_file($path . '/' . $file, $relative_path . $file);
                }
            }
        }
        closedir($dir);
    }

    // Método para mostrar la vista de plugins (Servidor y Cliente)
    // Método para mostrar la lista de plugins
    public function plugins()
    {
        $data['title']     = 'Plugins';
        $data['user']      = $this->user;
        $data['menu']      = $this->menu;
        $data['subMenu']   = $this->subMenu;
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma
        $data['plugins']   = $this->get_plugins();
        $this->load->view('templates/header', $data);
        $this->load->view('templates/sidebar', $data);
        $this->load->view('templates/topbar', $data);
        $this->load->view('admin/plugins', $data);
        $this->load->view('templates/footer');
    }

// Método para descargar un plugin desde la vista
    public function download_plugin($id)
    {
        // Obtener los datos del plugin por ID
        $plugin = $this->get_plugin_by_id($id);

        if (! $plugin) {
            show_404(); // Mostrar error 404 si el plugin no existe
        }

        // Verificar que la URL de descarga sea válida
        $url     = $plugin->download_link;
        $headers = @get_headers($url);

        if ($headers && strpos($headers[0], '200')) {
            // Descargar el archivo del plugin
            $data = file_get_contents($url);

            // Forzar la descarga del archivo
            $this->load->helper('download');
            force_download($plugin->name . '.zip', $data);

            // Guardar mensaje de éxito
            $this->session->set_flashdata('success', 'Plugin downloaded successfully.');
        } else {
            // Manejar el caso en que la URL de descarga no sea válida
            $this->session->set_flashdata('error', 'The plugin download link is not valid.');
        }

        redirect('admin/plugins'); // Redirigir a la página de plugins
    }

// Método para instalar un plugin
    public function install_plugin($id)
    {
        $plugin = $this->get_plugin_by_id($id);

        if (! $plugin) {
            show_404();
        }

        // Usa la versión más reciente disponible si existe
        $url = $plugin->download_link;
        if ($plugin->available_version && version_compare($plugin->available_version, $plugin->version, '>')) {
            $url = str_replace($plugin->version, $plugin->available_version, $url); // Actualizar la URL si es necesario
        }

        // Simula el proceso de instalación: descomprimir y mover a la ubicación correcta
        $zip_file    = file_get_contents($url);
        $plugin_path = FCPATH . 'plugins/' . $plugin->name;
        mkdir($plugin_path, 0755, true);
        $zip_file_path = $plugin_path . '/' . $plugin->name . '.zip';
        write_file($zip_file_path, $zip_file);

        // Suponiendo que se necesita descomprimir el archivo zip
        $zip = new ZipArchive;
        if ($zip->open($zip_file_path) === true) {
            $zip->extractTo($plugin_path);
            $zip->close();
            unlink($zip_file_path); // Eliminar el archivo zip después de la extracción

            // Marcar el plugin como instalado y actualizar la versión en la base de datos
            $this->db->where('id', $id);
            $this->db->update('plugins', ['installed' => 1, 'version' => $plugin->available_version ?: $plugin->version]);

            // Guardar mensaje de éxito
            $this->session->set_flashdata('success', 'Plugin installed successfully.');
        } else {
            $this->session->set_flashdata('error', 'Failed to install the plugin.');
        }

        redirect('admin/plugins');
    }

// Método para desinstalar un plugin
    public function uninstall_plugin($id)
    {
        $plugin = $this->get_plugin_by_id($id);

        if (! $plugin) {
            show_404();
        }

        // Simula el proceso de desinstalación: eliminar los archivos del plugin
        $plugin_path = FCPATH . 'plugins/' . $plugin->name;
        delete_files($plugin_path, true);
        rmdir($plugin_path);

        // Marcar el plugin como no instalado en la base de datos
        $this->db->where('id', $id);
        $this->db->update('plugins', ['installed' => 0]);

        // Guardar mensaje de éxito
        $this->session->set_flashdata('success', 'Plugin uninstalled successfully.');

        redirect('admin/plugins');
    }

    // Método para actualizar un plugin
    public function update_plugin($id)
    {
        // Obtener los datos del plugin por ID
        $plugin = $this->get_plugin_by_id($id);

        if (! $plugin) {
            show_404(); // Mostrar error 404 si el plugin no existe
        }

        // Verificar si hay una nueva versión disponible
        if ($plugin->available_version && version_compare($plugin->available_version, $plugin->version, '>')) {
            // Descargar la nueva versión del plugin
            $url     = $plugin->download_link;
            $headers = @get_headers($url);

            if ($headers && strpos($headers[0], '200')) {
                $zip_file      = file_get_contents($url);
                $plugin_path   = FCPATH . 'plugins/' . $plugin->name;
                $zip_file_path = $plugin_path . '/' . $plugin->name . '.zip';

                // Guardar el archivo zip
                write_file($zip_file_path, $zip_file);

                // Suponiendo que se necesita descomprimir el archivo zip
                $zip = new ZipArchive;
                if ($zip->open($zip_file_path) === true) {
                    $zip->extractTo($plugin_path);
                    $zip->close();
                    unlink($zip_file_path); // Eliminar el archivo zip después de la extracción
                }

                // Actualizar la versión instalada en la base de datos
                $this->db->where('id', $id);
                $this->db->update('plugins', ['version' => $plugin->available_version]);

                $this->session->set_flashdata('success', 'Plugin updated successfully.');
            } else {
                $this->session->set_flashdata('error', 'Failed to download the plugin update.');
            }
        } else {
            $this->session->set_flashdata('error', 'No valid update available.');
        }

        redirect('admin/plugins');
    }

    // Métodos privados para interactuar con la base de datos
    private function get_plugins()
    {
        $query = $this->db->get('plugins');
        return $query->result();
    }

    private function get_plugin_by_id($id)
    {
        $query = $this->db->get_where('plugins', ['id' => $id]);
        return $query->row();
    }

    public function get_installed_plugins()
    {
        $this->db->where('installed', 1);   // Filtrar por plugins instalados
        $query = $this->db->get('plugins'); // Obtener los resultados
        return $query->result();            // Retornar los plugins instalados
    }

    // Método para obtener el ID del diseño seleccionado
    public function get_selected_design_id()
    {
        $email = $this->user['email'];               // Obtener el email del usuario desde la sesión
        $user  = $this->User_model->getUser($email); // Llamar a getUser desde el User_model

        if ($user) {
            return $user['design_id']; // Retornar el design_id del usuario
        } else {
            return null; // Si no se encuentra el usuario o no tiene un diseño seleccionado
        }
    }

    // Método para guardar el ID del diseño seleccionado apra todos los que sean administradores
    public function save_design_selection()
    {
        $design_id = $this->input->post('design_id');

        // Filtrar usuarios con role_id igual a 1 y donde el design_id sea diferente al actual
        $this->db->where('role_id', 1);
        $this->db->where('design_id !=', $design_id); // Solo actualiza si el design_id es diferente
        $this->db->update('user', ['design_id' => $design_id]);

        if ($this->db->affected_rows() > 0) {
            echo json_encode(['success' => true]);
        } else {
            echo json_encode(['success' => true, 'message' => 'This design archive is already loaded.']);
        }
    }

}
