<?php
/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-09 16:54:52
 * @Last Modified by:   emo
 * @Last Modified time: 2024-10-27 22:03:57
 *
 * Adapted and updated from the code by https://github.com/sandhikagalih and
 * https://github.com/rafli-sss/wpu-login-pro, both under the MIT License.
 * All new code and additional functionalities created by Emo.
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */
defined('BASEPATH') or exit('No direct script access allowed');

class Auth extends CI_Controller
{
    public function __construct()
    {

        parent::__construct();

        // Obtener el idioma predeterminado desde la configuración de CodeIgniter
        $default_language = $this->config->item('language');

        $friendly_language = $default_language;
        // Mapeo de nombres amigables a códigos de idioma estándar
        $this->load->config('language');
        $language_directory      = $this->config->item('language_directory');
        $language_directory_name = $this->config->item('language_directory_name');

        // Obtener el nombre del directorio para el idioma
        $language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

        // Determinar el código de idioma
        $this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

        // Cargar el archivo de idioma desde el directorio correcto
        $this->lang->load('aplicacion_lang', $language_directory_name_value);

        $this->load->model('User_model');
        $this->load->model('Config_model');

        $this->load->library('user_agent');
        $this->load->model('Seo_model'); // Cargar el modelo SEO
                                         // Verificar si la clase Form_validation ya está cargada
        if (! isset($this->form_validation)) {
            $this->load->library('form_validation');
        }
        // Cargar HookManager y los plugins
        $this->load->library('HookManager');
        $this->hookmanager->load_plugins();
        //$this->hookmanager->load_specific_plugin('auto_ip_blocker');

    }

    public function index()
    {

        is_logged_in();
        $this->load->library('form_validation');
        $data['seo']       = $this->Seo_model->get_seo_settings();
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Ejecutar el hook y capturar el resultado
        $output = $this->hookmanager->run_hooks('view_login');

        // Pasar la salida del plugin a la vista como una variable
        $data['output'] = $output;

        $this->form_validation->set_rules('email', 'Email', 'trim|required|valid_email');
        $this->form_validation->set_rules('password', 'Password', 'trim|required');

        // Nueva regla para el checkbox de términos y condiciones
        $this->form_validation->set_rules('acceptTerms', 'Terms and Conditions', 'required');

        $this->Config_model->check_if_config_is_empty_and_restore_all_defaults();

        // Obtener el valor de activar_registro desde la tabla config
        $config                   = $this->Config_model->get_config_value('activar_registro');
        $data['activar_registro'] = ($config && isset($config['value'])) ? $config['value'] : 0;

        // Obtener el valor de la plantilla seleccionada desde la base de datos
        $plantilla_config = $this->Config_model->get_config_value('tipo_plantilla');
        $plantilla        = ($plantilla_config && isset($plantilla_config['value'])) ? $plantilla_config['value'] : 'normal'; // 'normal' como valor por defecto

        if (! $this->form_validation->run()) {
            $data['title'] = $this->lang->line('Login_Page');
            $this->load->view('templates/auth_header', $data);

            // Seleccionar la vista según la opción seleccionada
            switch ($plantilla) {
                case 'sidebar':

                    $this->load->view('auth/login_sidebar', $data);
                    break;
                case 'sidebar_right':
                    $this->load->view('auth/login_sidebar_right', $data);
                    break;
                case 'imagen transparente derecha':
                    $this->load->view('auth/login_imagen_derecha_transparente', $data);
                    break;
                case 'imagen transparente izquierda':
                    $this->load->view('auth/login_imagen_izquierda_transparente', $data);
                    break;
                // Agrega más casos según sea necesario
                default:
                    $this->load->view('auth/login', $data);
                    break;
            }

            $this->load->view('templates/auth_footer');
        } else {
            $this->_login();
        }
    }

/*	private function _login() {

// Validación de los datos de entrada
$this->form_validation->set_rules('email', 'Email', 'required|valid_email|trim');
$this->form_validation->set_rules('password', 'Password', 'required');

if ($this->form_validation->run() == false) {
$this->session->set_flashdata('alert-danger', validation_errors());
redirect('auth');
} else {
$email = $this->input->post('email', TRUE); // TRUE para aplicar XSS filtering
$password = $this->input->post('password', TRUE);

// Consulta el usuario usando un modelo
$user = $this->User_model->getUser($email);

if ($user) {
if ($user['is_active'] == 1) {
// Verificación de la contraseña
if (password_verify($password, $user['password'])) {
// Información del dispositivo
$ip = $this->input->ip_address();
$tipo_dispositivo = ($this->agent->is_mobile()) ? 'Mobile' : 'Desktop';
$sistema = $this->agent->platform();
$browser = $this->agent->browser() . ' ' . $this->agent->version();

// Actualiza el último acceso y la información del dispositivo
$this->User_model->updateLastAccess($email, $ip, $tipo_dispositivo, $sistema, $browser);

// Obtener el rol del usuario
$role = $this->User_model->getRole($user['role_id']);

// Crear los datos de la sesión
$data = [
'email' => $user['email'],
'role_id' => $user['role_id'],
'redirection' => $role['redirection'], // Agregar el campo de redirección
];

// Establecer la sesión
$this->session->set_userdata($data);

// Verificar si el modo de mantenimiento está activado y el usuario no es administrador
if ($this->config->item('maintenance_mode') && $user['role_id'] != 1) {
// Cargar los archivos de idioma de aplicacion
$this->lang->load('aplicacion_lang', $this->config->item('language'));

// Pasar las líneas de idioma a la vista
$data['maintenance_title'] = $this->lang->line('maintenance_title');
$data['maintenance_message'] = $this->lang->line('maintenance_message');
$data['maintenance_signature'] = $this->lang->line('maintenance_signature');
$data['lang_code'] = $this->lang_code;

// Destruir la sesión y mostrar la página de mantenimiento
$this->session->sess_destroy();
$this->load->view('maintenance', $data);
return;
}

// Redireccionar según el rol
if (!empty($role['redirection'])) {
redirect($role['redirection']);
} else {
redirect('user'); // Redirige a una ruta predeterminada si no hay redirección específica
}
} else {
$this->session->set_flashdata('alert-danger', $this->lang->line('Wrong_Password'));
$ip_address = $_SERVER['REMOTE_ADDR'];
log_message('error', 'Attempted access with invalid password from IP address: ' . $ip_address);
redirect('auth');
}
} else {
$this->session->set_flashdata('alert-danger', $this->lang->line('This email has not been activated!'));
redirect('auth');
}
} else {
$this->session->set_flashdata('alert-danger', $this->lang->line('Email is not registered!'));
$ip_address = $_SERVER['REMOTE_ADDR'];
log_message('error', 'Attempted access with unregistered email from IP address: ' . $ip_address);

redirect('auth');
}
}
}
 */

    private function _login()
    {
        // Verificar IP antes de intentar login
        $this->hookmanager->run_hooks('pre_login');
        $this->load->library('form_validation');
        // Validación de los datos de entrada
        $this->form_validation->set_rules('email', 'Email', 'required|valid_email|trim');
        $this->form_validation->set_rules('password', 'Password', 'required');

        if ($this->form_validation->run() == false) {
            $this->session->set_flashdata('alert-danger', validation_errors());
            redirect('auth');
        } else {
            $email    = $this->input->post('email', true); // TRUE para aplicar XSS filtering
            $password = $this->input->post('password', true);

            // Consulta el usuario usando un modelo
            $user = $this->User_model->getUser($email);

            if ($user) {
                if ($user['is_active'] == 1) {
                    // Verificación de la contraseña
                    if (password_verify($password, $user['password'])) {
                        // Información del dispositivo
                        $ip               = $this->input->ip_address();
                        $tipo_dispositivo = ($this->agent->is_mobile()) ? 'Mobile' : 'Desktop';
                        $sistema          = $this->agent->platform();
                        $browser          = $this->agent->browser() . ' ' . $this->agent->version();

                        // Actualiza el último acceso y la información del dispositivo
                        $this->User_model->updateLastAccess($email, $ip, $tipo_dispositivo, $sistema, $browser);

                        // Obtener el rol del usuario
                        $role = $this->User_model->getRole($user['role_id']);

                        // Crear los datos de la sesión
                        $data = [
                            'email'       => $user['email'],
                            'role_id'     => $user['role_id'],
                            'redirection' => $role['redirection'], // Agregar el campo de redirección
                        ];

                        // Establecer la sesión
                        $this->session->set_userdata($data);

                        // Verificar si el modo de mantenimiento está activado y el usuario no es administrador
                        if ($this->config->item('maintenance_mode') && $user['role_id'] != 1) {
                            // Cargar los archivos de idioma de aplicación
                            $this->lang->load('aplicacion_lang', $this->config->item('language'));

                            // Pasar las líneas de idioma a la vista
                            $data['maintenance_title']     = $this->lang->line('maintenance_title');
                            $data['maintenance_message']   = $this->lang->line('maintenance_message');
                            $data['maintenance_signature'] = $this->lang->line('maintenance_signature');
                            $data['lang_code']             = $this->lang_code;

                            // Destruir la sesión y mostrar la página de mantenimiento
                            $this->session->sess_destroy();
                            $this->load->view('maintenance', $data);
                            return;
                        }

                        // Redireccionar según el rol
                        if (! empty($role['redirection'])) {
                            redirect($role['redirection']);
                        } else {
                            redirect('user'); // Redirige a una ruta predeterminada si no hay redirección específica
                        }
                    } else {
                        // Ejecución del hook en caso de login fallido
                        $this->hookmanager->run_hooks('login_failed');

                        $this->session->set_flashdata('alert-danger', $this->lang->line('Wrong_Password'));
                        $ip_address = $_SERVER['REMOTE_ADDR'];
                        log_message('error', 'Attempted access with invalid password from IP address: ' . $ip_address);
                        redirect('auth');
                    }
                } else {
                    $this->session->set_flashdata('alert-danger', $this->lang->line('This email has not been activated!'));
                    redirect('auth');
                }
            } else {
                $this->session->set_flashdata('alert-danger', $this->lang->line('Email is not registered!'));
                $ip_address = $_SERVER['REMOTE_ADDR'];
                log_message('error', 'Attempted access with unregistered email from IP address: ' . $ip_address);

                // Ejecutar el hook en caso de login fallido
                $this->hookmanager->run_hooks('login_failed');

                redirect('auth');
            }
        }
    }

    public function registration()
    {
        // Verificar si el usuario ya está logueado
        is_logged_in();
        $this->load->library('form_validation');
        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // Verificar el estado de la configuración 'activar_registro'
        $activar_registro = $this->Config_model->get_config_value('activar_registro');

        // Verificar la configuración 'default_activation'
        $default_activation_config = $this->Config_model->get_config_value('default_activation');
        $default_activation        = intval($default_activation_config['value']); // Convertir a entero para asegurar la comparación

        // var_dump($activar_registro);
        // die(); // O exit();

        if ($activar_registro['value'] === '0') {
            $this->session->set_flashdata('alert-danger', 'User registration is currently disabled.');
            redirect('auth');
        }

        // Verificar el captcha
        $user_captcha   = $this->input->post('captcha');
        $captcha_answer = $this->input->post('captcha_answer');

        if ($user_captcha !== $captcha_answer) {
            // Manejar el error de captcha
            $this->session->set_flashdata('alert-danger', 'Incorrect CAPTCHA answer.');
            redirect('auth/registration');
        }

        // Validación de los datos de entrada
        $this->form_validation->set_rules('name', 'Name', 'required|trim');
        $this->form_validation->set_rules('email', 'Email', 'required|trim|valid_email|is_unique[user.email]', [
            'is_unique' => 'Email has already been registered!',
        ]);
        $this->form_validation->set_rules('password1', 'Password', 'required|trim|min_length[6]|matches[password2]', [
            'matches'    => 'Passwords do not match!',
            'min_length' => 'Password too short!',
        ]);
        $this->form_validation->set_rules('password2', 'Password Confirmation', 'required|trim|matches[password1]');

        if (! $this->form_validation->run()) {
            $data['title'] = 'User Registration';
            $this->load->view('templates/auth_header', $data);
            $this->load->view('auth/registration');
            $this->load->view('templates/auth_footer');
        } else {
            $email = $this->input->post('email', true); // TRUE para aplicar XSS filtering
            $data  = [
                'name'      => htmlspecialchars($this->input->post('name', true), ENT_QUOTES, 'UTF-8'),
                'email'     => htmlspecialchars($email, ENT_QUOTES, 'UTF-8'),
                'image'     => 'default.png',
                'password'  => password_hash($this->input->post('password1'), PASSWORD_DEFAULT),
                'role_id'   => 2,
                'is_active' => $default_activation, // Asignar directamente el valor de default_activation
                'date_created' => date('Y-m-d H:i:s'),
            ];

            // Preparar token
            $token = base64_encode(random_bytes(32));

            if ($default_activation == 0) {
                $user_token = [
                    'email'        => $email,
                    'token'        => htmlspecialchars($token, ENT_QUOTES, 'UTF-8'),
                    'date_created' => date('Y-m-d H:i:s'),
                ];
                $this->User_model->insertToken($user_token);
                $this->_sendEmail($token, 'verify');
                $this->session->set_flashdata('alert-success', $this->lang->line('auth_account_created_email'));
            } else {
                // No enviar correo de verificación
                $data['is_active'] = 1;
                $this->session->set_flashdata('alert-success', $this->lang->line('auth_account_created_login'));
            }

            $this->User_model->addUser($data);

            redirect('auth');
        }
    }

    /**
     * Sends an email.
     *
     * @param      <type>  $token  The token
     * @param      string  $type   The type
     *
     * @return     bool    ( description_of_the_return_value )
     */
    private function _sendEmail($token, $type)
    {
        $this->load->library('email');
        $this->email->set_mailtype("html");

        // Obtener la base_url y extraer solo el dominio
        $base_url   = base_url();
        $parsed_url = parse_url($base_url);
        $domain     = isset($parsed_url['host']) ? $parsed_url['host'] : '';

        // Enviar el correo de verificación o restablecimiento al usuario
        $this->email->from('noreply@' . $domain, $this->lang->line('Account Verification') . ': ' . $domain);
        $this->email->to($this->input->post('email'));
        $this->email->set_newline("\r\n"); //añadido para que funcione en php8
        $this->email->set_crlf("\r\n");    //añadido para que funcione en php8

        if ($type == 'verify') {
            $this->email->subject($this->lang->line('Account Verification'));
            $this->email->message('<h1 style="text-align: center;">' . $this->lang->line('Click this button to verify your account') . '</h1>
            <br />
            <div style="width:100%;display:flex;align-items:center;justify-content:center">
            <a style="width:200px;height:100px;background-color:#4E73DF;padding:10px 25px;margin:auto;text-decoration:none;color:white;border: 3px solid black;border-radius:10px;font-size:24px;font-weight:bold;text-align:center;line-height:100px;" href="' . base_url() . 'auth/verify?email=' . $this->input->post('email') . '&token=' . urlencode($token) . '">
            ' . $this->lang->line('Verify') . '
            </a>
            </div>');
        } else if ($type == 'forgot') {
            $this->email->subject($this->lang->line('Reset Password'));
            $this->email->message('<h1 style="text-align: center;">' . $this->lang->line('Click this button to reset your password') . '</h1>
            <br />
            <div style="width:100%;display:flex;align-items:center;justify-content:center">
            <a style="width:200px;height:100px;background-color:#4E73DF;padding:10px 25px;margin:auto;text-decoration:none;color:white;border: 3px solid black;border-radius:10px;font-size:24px;font-weight:bold;text-align:center;line-height:100px;" href="' . base_url() . 'auth/resetpassword?email=' . $this->input->post('email') . '&token=' . urlencode($token) . '">
            ' . $this->lang->line('Reset') . '
            </a>
            </div>');
        }

        // Enviar el correo al usuario
        if ($this->email->send()) {
            // Si el tipo es "verify", enviar el correo al administrador
            if ($type == 'verify') {
                // Obtener el primer email con role_id = 1 de la tabla user
                $this->load->database();
                $query = $this->db->select('email')
                    ->where('role_id', 1)
                    ->order_by('id', 'ASC')
                    ->limit(1)
                    ->get('user');

                if ($query->num_rows() > 0) {
                    $admin_email = $query->row()->email;

                                               // Preparar y enviar el correo al administrador
                    $this->email->clear(true); // Limpiar la configuración del correo anterior
                    $this->email->from('noreply@' . $domain, 'Nuevo Registro en ' . $domain);
                    $this->email->to($admin_email);
                    $this->email->subject('Nuevo Usuario Registrado');
                    $this->email->message('Se ha registrado un nuevo usuario con el nombre: ' . $this->input->post('name') . ' y el email: ' . $this->input->post('email'));

                    if (! $this->email->send()) {
                        echo $this->email->print_debugger();
                        die;
                    }
                } else {
                    // Manejar el caso en que no se encuentre ningún usuario con role_id=1
                    log_message('error', 'No se encontró ningún usuario con role_id=1 para enviar la notificación.');
                }
            }
            return true;
        } else {
            echo $this->email->print_debugger();
            die;
        }
    }

    /**
     * Sends an email sidebar.
     *
     * @param      <type>  $token          The token
     * @param      string  $type           The type
     * @param      string  $email_address  The email address
     *
     * @return     bool    ( description_of_the_return_value )
     */
    private function _sendEmail_sidebar($token, $type, $email_address)
    {
        $this->load->library('email');
        $this->email->set_mailtype("html");

        // Obtener la base_url y extraer solo el dominio
        $base_url   = base_url();
        $parsed_url = parse_url($base_url);
        $domain     = isset($parsed_url['host']) ? $parsed_url['host'] : '';

        log_message('debug', 'Email address in _sendEmail: ' . $email_address); // Depuración

        // Verificar si el correo electrónico es válido
        if (! filter_var($email_address, FILTER_VALIDATE_EMAIL)) {
            log_message('error', 'Invalid email address: ' . $email_address);
            return false;
        }

        // Configurar y enviar el correo de verificación o restablecimiento al usuario
        $this->email->from('noreply@' . $domain, $this->lang->line('Account Verification') . ': ' . $domain);
        $this->email->to($email_address);

        $this->email->set_newline("\r\n"); // Añadido para que funcione en PHP 8
        $this->email->set_crlf("\r\n");    // Añadido para que funcione en PHP 8

        if ($type == 'verify') {
            $this->email->subject($this->lang->line('Account Verification'));
            $this->email->message('<h1 style="text-align: center;">' . $this->lang->line('Click this button to verify your account') . '</h1>
        <br />
        <div style="width:100%;display:flex;align-items:center;justify-content:center">
        <a style="width:200px;height:100px;background-color:#4E73DF;padding:10px 25px;margin:auto;text-decoration:none;color:white;border: 3px solid black;border-radius:10px;font-size:24px;font-weight:bold;text-align:center;line-height:100px;" href="' . base_url() . 'auth/verify?email=' . $email_address . '&token=' . urlencode($token) . '">
        ' . $this->lang->line('Verify') . '
        </a>
        </div>');
        } else if ($type == 'forgot') {
            $this->email->subject($this->lang->line('Reset Password'));
            $this->email->message('<h1 style="text-align: center;">' . $this->lang->line('Click this button to reset your password') . '</h1>
        <br />
        <div style="width:100%;display:flex;align-items:center;justify-content:center">
        <a style="width:200px;height:100px;background-color:#4E73DF;padding:10px 25px;margin:auto;text-decoration:none;color:white;border: 3px solid black;border-radius:10px;font-size:24px;font-weight:bold;text-align:center;line-height:100px;" href="' . base_url() . 'auth/resetpassword?email=' . $email_address . '&token=' . urlencode($token) . '">
        ' . $this->lang->line('Reset') . '
        </a>
        </div>');
        }

        // Enviar el correo al usuario
        if ($this->email->send()) {
            log_message('debug', 'Email sent successfully to: ' . $email_address);

            // Si el tipo es "verify", enviar el correo al administrador
            if ($type == 'verify') {
                // Obtener el primer email con role_id = 1 de la tabla user
                $this->load->database();
                $query = $this->db->select('email')
                    ->where('role_id', 1)
                    ->order_by('id', 'ASC')
                    ->limit(1)
                    ->get('user');

                if ($query->num_rows() > 0) {
                    $admin_email = $query->row()->email;

                                               // Preparar y enviar el correo al administrador
                    $this->email->clear(true); // Limpiar la configuración del correo anterior
                    $this->email->from('noreply@' . $domain, 'Nuevo Registro en ' . $domain);
                    $this->email->to($admin_email);
                    $this->email->subject('Nuevo Usuario Registrado');
                    $this->email->message('Se ha registrado un nuevo usuario con el email: ' . $email_address);

                    if (! $this->email->send()) {
                        log_message('error', 'Email sending failed to admin: ' . $this->email->print_debugger());
                        return false;
                    } else {
                        log_message('debug', 'Notification email sent to admin: ' . $admin_email);
                    }
                } else {
                    // Manejar el caso en que no se encuentre ningún usuario con role_id=1
                    log_message('error', 'No se encontró ningún usuario con role_id=1 para enviar la notificación.');
                }
            }

            return true;
        } else {
            log_message('error', 'Email sending failed: ' . $this->email->print_debugger());
            return false;
        }
    }

    /**
     * verify user
     */
    public function verify()
    {
        $email = $this->input->get('email');
        $token = $this->input->get('token');

        $user = $this->User_model->getUser($email);

        if ($user) {
            $user_token = $this->User_model->getUserByToken($token);

            if ($user_token) {
                $token_created = strtotime($user_token['date_created']);

                if (time() - $token_created < (60 * 60 * 24)) {
                    $this->User_model->updateUser($email);
                    $this->User_model->deleteToken($email);

                    $this->session->set_flashdata('alert-success', $email . $this->lang->line('auth_activated'));
                    redirect('auth');
                } else {
                    $this->User_model->deleteUser($email);
                    $this->User_model->deleteToken($email);

                    $this->session->set_flashdata('alert-danger', $this->lang->line('Account activation failed! Token expired, please fill registration form again.'));
                    redirect('auth/registration');
                }
            } else {
                $this->session->set_flashdata('alert-danger', $this->lang->line('Account activation failed! Invalid token'));
                redirect('auth');
            }
        } else {
            $this->session->set_flashdata('alert-danger', $this->lang->line('Account activation failed! Wrong email'));
            redirect('auth');
        }
    }

    /**
     * logout app
     */
    public function logout()
    {

        if (! $this->session->userdata('email')) {
            redirect('auth');
        }

        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $this->session->unset_userdata('email');
        $this->session->unset_userdata('role_id');

        $this->session->set_flashdata('alert-success', $this->lang->line('You have been logged out!'));
        redirect('auth');
        $this->session->sess_destroy();
    }

    /**
     * Access blocked
     */
    public function blocked()
    {

        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        // redirect('auth');
        $data['title'] = $this->lang->line('Access Blocked');
        $this->load->view('templates/header', $data);
        $this->load->view('auth/blocked');
        $this->load->view('templates/footer', $data);
    }

    /**
     * forgot password
     */
    public function forgotPassword()
    {

        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $this->form_validation->set_rules('email', 'Email', 'trim|required|valid_email');

        if (! $this->form_validation->run()) {
            $data['title'] = $this->lang->line('Forgot Password');
            $this->load->view('templates/auth_header', $data);
            $this->load->view('auth/forgot-password');
            $this->load->view('templates/auth_footer');
        } else {
            $email = $this->input->post('email');
            $user  = $this->User_model->getActiveUser($email);

            if ($user) {
                $token      = base64_encode(random_bytes(32));
                $user_token = [
                    'email'        => $email,
                    'token'        => $token,
                    'date_created' => date('Y-m-d H:i:s'), // Formato DATETIME
                ];

                $this->User_model->insertToken($user_token);
                $this->_sendEmail($token, 'forgot');

                $this->session->set_flashdata('alert-success', $this->lang->line('Please check your email to reset your password!'));
                redirect('auth/forgotPassword');
            } else {
                $this->session->set_flashdata('alert-danger', $this->lang->line('Email is not registered or activated!'));
                redirect('auth/forgotPassword');
            }
        }
    }

    /**
     * Reset password
     */
    public function resetPassword()
    {

        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        $email = $this->input->get('email');
        $token = $this->input->get('token');
        $user  = $this->User_model->getUser($email, 'user_token');

        if ($user) {
            $user_token = $this->User_model->getUserByToken($token);

            if ($user_token) {
                $token_created = strtotime($user_token['date_created']);

                if (time() - $token_created < (60 * 60 * 24)) {
                    $this->session->set_userdata('reset_email', $email);
                    $this->changePassword();
                } else {
                    $this->session->set_flashdata('alert-danger', $this->lang->line('Token expired! Please re-enter your email for new token'));
                    redirect('auth/forgotPassword');
                }
            } else {
                $this->session->set_flashdata('alert-danger', $this->lang->line('Reset password failed! Invalid Token'));
                redirect('auth/forgotPassword');
            }
        } else {
            $this->session->set_flashdata('alert-danger', $this->lang->line('Reset password failed! Invalid Email'));
            redirect('auth/forgotPassword');
        }
    }

    /**
     * Change password
     */
    public function changePassword()
    {

        $data['lang']      = $this->lang->language; // Pasar el array de idioma a la vista
        $data['lang_code'] = $this->lang_code;      // Código del idioma

        if (! $this->session->userdata('reset_email')) {
            redirect('auth');
            die();
        }

        $this->form_validation->set_rules('password1', 'Password', 'trim|required|min_length[8]|matches[password2]');
        $this->form_validation->set_rules('password2', 'Repeat Password', 'trim|required|min_length[8]|matches[password1]');

        if (! $this->form_validation->run()) {
            $data['title'] = $this->lang->line('Change Password');
            $this->load->view('templates/auth_header', $data);
            $this->load->view('auth/change-password');
            $this->load->view('templates/auth_footer');
        } else {
            $password = password_hash($this->input->post('password1'), PASSWORD_DEFAULT);
            $email    = $this->session->userdata('reset_email');

            $this->User_model->updatePassword($password, $email);
            $this->User_model->deleteToken($email);
            $this->session->unset_userdata('reset_email');

            $this->session->set_flashdata('alert-success', $this->lang->line('Password has been changed! Please Login'));
            redirect('auth');
        }
    }

/*	public function login_sidebar() {
$this->form_validation->set_rules('login_email', 'Email', 'trim|required|valid_email');
$this->form_validation->set_rules('login_password', 'Password', 'trim|required');

if ($this->form_validation->run() == false) {
$response = array('status' => 'error', 'message' => validation_errors());
echo json_encode($response);
} else {
$email = $this->input->post('login_email', TRUE);
$password = $this->input->post('login_password', TRUE);

$user = $this->User_model->getUser($email);

if ($user) {
if ($user['is_active'] == 1) {
if (password_verify($password, $user['password'])) {
$ip = $this->input->ip_address();
$tipo_dispositivo = ($this->agent->is_mobile()) ? 'Mobile' : 'Desktop';
$sistema = $this->agent->platform();
$browser = $this->agent->browser() . ' ' . $this->agent->version();

$this->User_model->updateLastAccess($email, $ip, $tipo_dispositivo, $sistema, $browser);

$role = $this->User_model->getRole($user['role_id']);

$data = [
'email' => $user['email'],
'role_id' => $user['role_id'],
'redirection' => $role['redirection'],
];

$this->session->set_userdata($data);

// Verificar si el modo de mantenimiento está activado y el usuario no es administrador
if ($this->config->item('maintenance_mode') && $user['role_id'] != 1) {
// Cargar los archivos de idioma de aplicacion
$this->lang->load('aplicacion_lang', $this->config->item('language'));

// Pasar las líneas de idioma a la vista
$data['maintenance_title'] = $this->lang->line('maintenance_title');
$data['maintenance_message'] = $this->lang->line('maintenance_message');
$data['maintenance_signature'] = $this->lang->line('maintenance_signature');
$data['lang_code'] = $this->lang_code;

// Destruir la sesión y mostrar la página de mantenimiento
$this->session->sess_destroy();
$this->load->view('maintenance', $data);
return;
}
$response = array('status' => 'success', 'message' => 'Login successful!', 'redirect' => !empty($role['redirection']) ? base_url($role['redirection']) : base_url('user'));
echo json_encode($response);
} else {
$response = array('status' => 'error', 'message' => $this->lang->line('Wrong_Password'));
$ip_address = $_SERVER['REMOTE_ADDR'];
log_message('error', 'Attempted access with invalid password from IP address: ' . $ip_address);

echo json_encode($response);
}
} else {
$response = array('status' => 'error', 'message' => $this->lang->line('This email has not been activated!'));
echo json_encode($response);
}
} else {
$response = array('status' => 'error', 'message' => $this->lang->line('Email is not registered!'));
$ip_address = $_SERVER['REMOTE_ADDR'];
log_message('error', 'Attempted access with unregistered email from IP address: ' . $ip_address);
echo json_encode($response);

}
}
}*/

    public function login_sidebar()
    {
        $this->hookmanager->run_hooks('pre_login');

        $this->form_validation->set_rules('login_email', 'Email', 'trim|required|valid_email');
        $this->form_validation->set_rules('login_password', 'Password', 'trim|required');

        // Nueva regla para el checkbox de términos y condiciones
        $this->form_validation->set_rules('acceptTerms', 'Terms and Conditions', 'required');

        if ($this->form_validation->run() == false) {
            $response = ['status' => 'error', 'message' => validation_errors()];
            echo json_encode($response);
        } else {
            $email    = $this->input->post('login_email', true);
            $password = $this->input->post('login_password', true);

            $user = $this->User_model->getUser($email);

            if ($user) {
                if ($user['is_active'] == 1) {
                    if (password_verify($password, $user['password'])) {
                        $ip               = $this->input->ip_address();
                        $tipo_dispositivo = ($this->agent->is_mobile()) ? 'Mobile' : 'Desktop';
                        $sistema          = $this->agent->platform();
                        $browser          = $this->agent->browser() . ' ' . $this->agent->version();

                        $this->User_model->updateLastAccess($email, $ip, $tipo_dispositivo, $sistema, $browser);

                        $role = $this->User_model->getRole($user['role_id']);

                        $data = [
                            'email'       => $user['email'],
                            'role_id'     => $user['role_id'],
                            'redirection' => $role['redirection'],
                        ];

                        $this->session->set_userdata($data);

                        // Verificar si el modo de mantenimiento está activado y el usuario no es administrador
                        if ($this->config->item('maintenance_mode') && $user['role_id'] != 1) {
                            // Cargar los archivos de idioma de aplicacion
                            $this->lang->load('aplicacion_lang', $this->config->item('language'));

                            // Pasar las líneas de idioma a la vista
                            $data['maintenance_title']     = $this->lang->line('maintenance_title');
                            $data['maintenance_message']   = $this->lang->line('maintenance_message');
                            $data['maintenance_signature'] = $this->lang->line('maintenance_signature');
                            $data['lang_code']             = $this->lang_code;

                            // Destruir la sesión y mostrar la página de mantenimiento
                            $this->session->sess_destroy();
                            $this->load->view('maintenance', $data);
                            return;
                        }
                        $response = ['status' => 'success', 'message' => 'Login successful!', 'redirect' => ! empty($role['redirection']) ? base_url($role['redirection']) : base_url('user')];
                        echo json_encode($response);
                    } else {
                        // Ejecución del hook en caso de login fallido
                        $this->hookmanager->run_hooks('login_failed');

                        $response   = ['status' => 'error', 'message' => $this->lang->line('Wrong_Password')];
                        $ip_address = $_SERVER['REMOTE_ADDR'];
                        log_message('error', 'Attempted access with invalid password from IP address: ' . $ip_address);

                        echo json_encode($response);
                    }
                } else {
                    $response = ['status' => 'error', 'message' => $this->lang->line('This email has not been activated!')];
                    echo json_encode($response);
                }
            } else {
                $response   = ['status' => 'error', 'message' => $this->lang->line('Email is not registered!')];
                $ip_address = $_SERVER['REMOTE_ADDR'];
                log_message('error', 'Attempted access with unregistered email from IP address: ' . $ip_address);

                // Ejecutar el hook en caso de login fallido
                $this->hookmanager->run_hooks('login_failed');

                echo json_encode($response);
            }
        }
    }

    public function forgotPassword_sidebar()
    {
        $this->form_validation->set_rules('forgot_email', 'Email', 'trim|required|valid_email');

        if ($this->form_validation->run() == false) {
            $response = ['status' => 'error', 'message' => validation_errors()];
            echo json_encode($response);
        } else {
            $email = $this->input->post('forgot_email');
            log_message('debug', 'Email for password reset: ' . $email); // Depuración

            $user = $this->User_model->getActiveUser($email);

            if ($user) {
                $token      = base64_encode(random_bytes(32));
                $user_token = [
                    'email'        => $email,
                    'token'        => $token,
                    'date_created' => date('Y-m-d H:i:s'), // Formato DATETIME
                ];

                $this->User_model->insertToken($user_token);

                // Agregar registro de depuración
                log_message('debug', 'Token generated and inserted: ' . print_r($user_token, true));

                if ($this->_sendEmail_sidebar($token, 'forgot', $email)) {
                    $response = ['status' => 'success', 'message' => $this->lang->line('Please check your email to reset your password!'), 'redirect' => base_url('auth')];
                } else {
                    $response = ['status' => 'error', 'message' => 'Failed to send email. Please try again later.'];
                }

                echo json_encode($response);
            } else {
                $response = ['status' => 'error', 'message' => $this->lang->line('Email is not registered or activated!')];
                echo json_encode($response);
            }
        }
    }

    public function registration_sidebar()
    {
        is_logged_in();
        $this->load->library('form_validation');

        // Verificar el captcha
        $user_captcha   = $this->input->post('captcha');
        $captcha_answer = $this->input->post('captcha_answer');

        if ($user_captcha !== $captcha_answer) {
            // Manejar el error de captcha
            //$this->session->set_flashdata('alert-danger', 'Incorrect CAPTCHA answer.');
            $response = ['status' => 'error', 'message' => 'Incorrect CAPTCHA answer.'];
            echo json_encode($response);
            //redirect('auth/registration');
            return;
        }

        $this->form_validation->set_rules('reg_name', 'Name', 'required|trim');
        $this->form_validation->set_rules('reg_email', 'Email', 'required|trim|valid_email|is_unique[user.email]', [
            'is_unique' => 'Email has already been registered!',
        ]);
        $this->form_validation->set_rules('reg_password1', 'Password', 'required|trim|min_length[6]|matches[reg_password2]', [
            'matches'    => 'Passwords do not match!',
            'min_length' => 'Password too short!',
        ]);
        $this->form_validation->set_rules('reg_password2', 'Password Confirmation', 'required|trim|matches[reg_password1]');

        if ($this->form_validation->run() == false) {
            $response = ['status' => 'error', 'message' => validation_errors()];
            echo json_encode($response);
        } else {
            $email = $this->input->post('reg_email', true);
            log_message('debug', 'Email for registration: ' . $email); // Depuración

            $activar_registro = $this->Config_model->get_config_value('activar_registro');

            if ($activar_registro['value'] === '0') {

                $response = ['status' => 'error', 'message' => $this->lang->line('User registration is currently disabled.')];
                echo json_encode($response);

                return;
            }

            $default_activation_config = $this->Config_model->get_config_value('default_activation');
            $default_activation        = intval($default_activation_config['value']);

            $data = [
                'name'         => htmlspecialchars($this->input->post('reg_name', true), ENT_QUOTES, 'UTF-8'),
                'email'        => htmlspecialchars($email, ENT_QUOTES, 'UTF-8'),
                'image'        => 'default.png',
                'password'     => password_hash($this->input->post('reg_password1'), PASSWORD_DEFAULT),
                'role_id'      => 2,
                'is_active'    => $default_activation,
                'date_created' => date('Y-m-d H:i:s'),
            ];

            if ($default_activation == 0) {
                $token      = base64_encode(random_bytes(32));
                $user_token = [
                    'email'        => $email,
                    'token'        => htmlspecialchars($token, ENT_QUOTES, 'UTF-8'),
                    'date_created' => date('Y-m-d H:i:s'),
                ];
                $this->User_model->insertToken($user_token);
                $this->_sendEmail_sidebar($token, 'verify', $email);
                $response = ['status' => 'success', 'message' => $this->lang->line('auth_account_created_email'), 'redirect' => base_url('auth')];
            } else {
                $data['is_active'] = 1;
                $response          = ['status' => 'success', 'message' => $this->lang->line('auth_account_created_login'), 'redirect' => base_url('auth')];
            }

            $this->User_model->addUser($data);
            echo json_encode($response);
        }
    }

}
