<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-26 12:30:29
 * website: https://loginbasicoemo.com
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') OR exit('No direct script access allowed');

class Crud extends CI_Controller {
	private $user, $menu, $subMenu;

	public function __construct() {
		parent::__construct();
		is_logged_in();

		// Obtener los datos del usuario desde la sesión
		$this->user = $this->session->userdata();
		//verificar es admin si no redirccionar
		if ($this->user['role_id'] != 1) {
			// Redirigir a la página por defecto del usuario
			redirect($this->session->userdata('redirection'));
		}
		// Verificar si los datos del usuario están disponibles
		if (empty($this->user)) {
			redirect('auth'); // Redirige al login si no hay datos de usuario
		}

		// Obtener el email del usuario
		$email = $this->user['email'];
		$this->load->model('User_model');

		// Obtener el idioma del usuario desde la base de datos

		$default_language = $this->User_model->get_user_language($email);

		$friendly_language = $default_language;

		// Mapeo de nombres amigables a códigos de idioma estándar
		$this->load->config('language');
		$language_directory = $this->config->item('language_directory');
		$language_directory_name = $this->config->item('language_directory_name');

		// Obtener el nombre del directorio para el idioma
		$language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

		// Determinar el código de idioma
		$this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

		// Cargar el archivo de idioma desde el directorio correcto
		$this->lang->load('aplicacion_lang', $language_directory_name_value);

		$this->load->model('Config_model');
		$this->user = $this->User_model->getUser($this->session->userdata('email'));
		$this->menu = $this->User_model->getSideMenu($this->session->userdata('role_id'));
		$this->subMenu = $this->User_model->getSideSubMenu($this->menu);
		$this->load->model('Crud_model');
		$this->load->helper(array('url', 'form'));

		$this->load->model('Seo_model'); // Cargar el modelo SEO
		$this->data['seo'] = $this->Seo_model->get_seo_settings(); // Obtener configuraciones SEO

		// Verificar si la clase Form_validation ya está cargada
		if (!isset($this->form_validation)) {
			$this->load->library('form_validation');
		}

	}

	/**
	 * Crud user list
	 *
	 * @param      int   $offset  The offset
	 */
	public function index($offset = 0) {
		$limit = 10;
		$data['users'] = $this->Crud_model->get_all_users($limit, $offset);

		// Convertir las fechas de UTC a la hora local del usuario
		foreach ($data['users'] as &$user) {
			// Convertir la fecha de creación
			$user['date_created'] = $this->convertToUserTimezone($user['date_created']);

			// Convertir el último acceso (si está disponible en los datos del usuario)
			$user['ultimo_acceso'] = isset($user['ultimo_acceso']) ? $this->convertToUserTimezone($user['ultimo_acceso']) : 'Fecha no disponible';
		}

		$this->load->library('pagination');
		$config['base_url'] = base_url('crud/index');
		$config['total_rows'] = $this->Crud_model->count_all_users();
		$config['per_page'] = $limit;
		$config['full_tag_open'] = '<nav><ul class="pagination justify-content-center">';
		$config['full_tag_close'] = '</ul></nav>';
		$config['first_link'] = 'Primero';
		$config['last_link'] = 'Último';
		$config['first_tag_open'] = '<li class="page-item">';
		$config['first_tag_close'] = '</li>';
		$config['prev_link'] = '&laquo;';
		$config['prev_tag_open'] = '<li class="page-item">';
		$config['prev_tag_close'] = '</li>';
		$config['next_link'] = '&raquo;';
		$config['next_tag_open'] = '<li class="page-item">';
		$config['next_tag_close'] = '</li>';
		$config['last_tag_open'] = '<li class="page-item">';
		$config['last_tag_close'] = '</li>';
		$config['cur_tag_open'] = '<li class="page-item active"><a class="page-link" href="#">';
		$config['cur_tag_close'] = '</a></li>';
		$config['num_tag_open'] = '<li class="page-item">';
		$config['num_tag_close'] = '</li>';
		$config['disabled_tag_open'] = '<li class="page-item disabled"><span class="page-link">';
		$config['disabled_tag_close'] = '</span></li>';
		$config['attributes'] = array('class' => 'page-link');
		$this->pagination->initialize($config);
		$data['pagination'] = $this->pagination->create_links();
		$data['title'] = $this->lang->line('Edit Users');
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$this->load->view('templates/header', $data);
		$this->load->view('templates/sidebar', $data);
		$this->load->view('templates/topbar', $data);
		$this->load->view('admin/crud_list', $data);
		$this->load->view('templates/footer');
	}

	/**
	 * Crud create user
	 */
	public function create() {
		$data['title'] = 'Crear usuario';
		//$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		// Obtener la configuración de idiomas
		$this->load->config('language');
		$data['language_directory'] = $this->config->item('language_directory');
		$data['language_directory_name'] = $this->config->item('language_directory_name');

		// Obtener los roles disponibles desde el modelo
		$this->load->model('Crud_model'); // Asegúrate de tener un modelo para los roles
		$data['roles'] = $this->Crud_model->get_all_roles();

		$this->load->view('templates/header', $data);
		$this->load->view('templates/sidebar', $data);
		$this->load->view('templates/topbar', $data);
		$this->load->view('admin/crud_form', $data);
		$this->load->view('templates/footer');
	}

	/**
	 * Crud save user
	 */
	public function store() {
		$this->form_validation->set_rules('name', 'Name', 'required');
		$this->form_validation->set_rules('email', 'Email', 'required|valid_email');
		$this->form_validation->set_rules('password', 'Password', 'required');
		if ($this->form_validation->run() == FALSE) {
			$this->load->view('crud_form');
		} else {
			$data = array(
				'name' => $this->input->post('name'),
				'email' => $this->input->post('email'),
				'image' => $this->input->post('image'),
				'password' => password_hash($this->input->post('password'), PASSWORD_BCRYPT),
				'role_id' => $this->input->post('role_id'),
				'is_active' => $this->input->post('is_active'),
				'date_created' => date('Y-m-d H:i:s'),
				'ultimo_acceso' => date('Y-m-d H:i:s'),
			);
			$this->Crud_model->insert_user($data);
			redirect('crud');
		}
	}

	/**
	 * Crud edit user by id
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function edit($id) {
		$data['user'] = $this->Crud_model->get_user_by_id($id);
		$data['title'] = 'Editar usuario';
		// $data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		// Obtener la configuración de idiomas
		$this->load->config('language');
		$data['language_directory'] = $this->config->item('language_directory');
		$data['language_directory_name'] = $this->config->item('language_directory_name');

		// Obtener los roles disponibles desde el modelo
		$this->load->model('Crud_model'); // Asegúrate de tener un modelo para los roles
		$data['roles'] = $this->Crud_model->get_all_roles();

		$this->load->view('templates/header', $data);
		$this->load->view('templates/sidebar', $data);
		$this->load->view('templates/topbar', $data);
		$this->load->view('admin/crud_form', $data);
		$this->load->view('templates/footer');
	}

	/**
	 * Crud Updates user the given identifier.
	 *
	 * @param      int   $id     The identifier
	 */
	public function update($id) {
		$this->form_validation->set_rules('name', 'Name', 'required');
		$this->form_validation->set_rules('email', 'Email', 'required|valid_email');

		if ($this->form_validation->run() == FALSE) {
			$data['user'] = $this->Crud_model->get_user_by_id($id);
			$this->load->view('crud_form', $data);
		} else {
			// Obtener el email actual del usuario antes de la actualización
			$current_user = $this->Crud_model->get_user_by_id($id);
			//$current_email = $current_user->email;
			$current_email = $current_user['email'];

			$new_email = $this->input->post('email');

			// Preparar los datos para la actualización
			$data = array(
				'name' => $this->input->post('name'),
				'email' => $new_email,
				'image' => $this->input->post('image'),
				'role_id' => $this->input->post('role_id'),
				'is_active' => $this->input->post('is_active'),
				'date_created' => date('Y-m-d H:i:s'),
				'language' => $this->input->post('language'),
			);

			// Actualizar el usuario en la base de datos
			$this->Crud_model->update_user($id, $data);

			// Verificar si el correo electrónico ha cambiado y si el usuario es el admin con id = 15
			if ($new_email !== $current_email && $id == 15) {
				$message = $this->lang->line('email_change_message');
				echo '<script type="text/javascript">
                    alert("' . $message . '");
                    window.location.href = "' . site_url('auth/logout') . '";
                  </script>';
				return;
			} else {
				// Si el correo electrónico no ha cambiado o no es el admin, redirigir a la página de CRUD
				redirect('crud');
			}
		}
	}

	/**
	 * Crud Deletes the given identifier.
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function delete($id) {
		$this->Crud_model->delete_user($id);
		redirect('crud');
	}

	/**
	 * Crud Export  csv data UTC
	 */
	public function export_csv_UTC() {
		$this->load->model('Crud_model');
		$users = $this->Crud_model->get_all_users1();
		$filename = 'usuarios_' . date('Ymd') . '.csv';

		// Establecer la zona horaria del servidor si es necesario
		//date_default_timezone_set('UTC');

		header('Content-Type: text/csv');
		header('Content-Disposition: attachment; filename="' . $filename . '"');
		$output = fopen('php://output', 'w');
		$header = array('ID', 'Nombre', 'Email', 'Imagen', 'Password', 'Rol', 'Activo', 'Fecha de Creación', 'Último Acceso');
		fputcsv($output, $header);

		foreach ($users as $user) {
			// Convertir las fechas a formato deseado (ejemplo: d-m-Y H:i:s)
			$user['date_created'] = date('d-m-Y H:i:s', strtotime($user['date_created']));
			$user['ultimo_acceso'] = date('d-m-Y H:i:s', strtotime($user['ultimo_acceso']));

			fputcsv($output, $user);
		}

		fclose($output);
		exit;
	}

	/**
	 * Crud export data
	 */
	public function export_csv() {
		$this->load->model('Crud_model');
		$users = $this->Crud_model->get_all_users1();
		$filename = 'users_' . date('Ymd') . '.csv'; // Nombre del archivo en inglés
		header('Content-Type: text/csv');
		header('Content-Disposition: attachment; filename="' . $filename . '"');
		$output = fopen('php://output', 'w');

		// Encabezados para el archivo CSV en inglés
		$header = array('ID', 'Name', 'Email', 'Image', 'Role', 'Active', 'Date Created', 'Last Access', 'Last Activity', 'Session Count', 'IP', 'Device Type', 'Operating System', 'Browser');
		fputcsv($output, $header);

		// Convertir las fechas de UTC a la hora local del usuario antes de escribir en el CSV
		foreach ($users as $user) {
			$converted_date_created = $this->convertToUserTimezone($user['date_created']);
			$converted_last_access = isset($user['ultimo_acceso']) ? $this->convertToUserTimezone($user['ultimo_acceso']) : '';
			$converted_last_activity = isset($user['ultima_actividad']) ? $this->convertToUserTimezone($user['ultima_actividad']) : '';
			// Construir una nueva fila para el CSV con las fechas convertidas
			$csv_data = array(
				$user['id'],
				$user['name'],
				$user['email'],
				$user['image'],
				/*      $user['password'], */
				$user['role_id'],
				$user['is_active'] ? 'Yes' : 'No',
				$converted_date_created,
				$converted_last_access,
				$converted_last_activity,
				$user['session_count'],
				$user['ip'],
				$user['tipo_dispositivo'],
				$user['sistema'],
				$user['browser'],
			);

			// Escribir la fila en el archivo CSV
			fputcsv($output, $csv_data);
		}

		fclose($output);
		exit;
	}

// Función para convertir una fecha de UTC a la hora local del usuario

	/**
	 * Conver utc time to user time
	 *
	 * @param      <type>  $utc_datetime  The utc datetime
	 *
	 * @return     string  ( description_of_the_return_value )
	 */
	private function convertToUserTimezone($utc_datetime) {
		if (empty($utc_datetime)) {
			return 'Fecha no disponible'; // Valor predeterminado si la fecha es null o está vacía
		}

		$utc_timezone = new DateTimeZone('UTC');
		$user_timezone = new DateTimeZone('Europe/Madrid'); // Cambia a la zona horaria del usuario

		try {
			$datetime = new DateTime($utc_datetime, $utc_timezone);
		} catch (Exception $e) {
			return 'Fecha no válida'; // Valor predeterminado si la fecha no es válida
		}

		$datetime->setTimezone($user_timezone);

		return $datetime->format('Y-m-d H:i:s'); // Formato de fecha deseado
	}

}
