<?php if (!defined('BASEPATH')) {
	exit('No direct script access allowed');
}

class Dbplugins extends CI_Controller {
	private $user, $menu, $subMenu;

	public function __construct() {
		parent::__construct();

		is_logged_in();

		// Obtener los datos del usuario desde la sesión
		$this->user = $this->session->userdata();

		// Verificar si los datos del usuario están disponibles
		if (empty($this->user)) {
			redirect('auth'); // Redirige al login si no hay datos de usuario
		}

		if ($this->user['role_id'] != 1) {
			// Redirigir a la página por defecto del usuario
			redirect($this->session->userdata('redirection'));
		}

		// Obtener el email del usuario
		$email = $this->user['email'];
		$this->load->model('User_model');

		// Obtener el idioma del usuario desde la base de datos

		$default_language = $this->User_model->get_user_language($email);

		$friendly_language = $default_language;

		// Mapeo de nombres amigables a códigos de idioma estándar
		$this->load->config('language');
		$language_directory = $this->config->item('language_directory');
		$language_directory_name = $this->config->item('language_directory_name');

		// Obtener el nombre del directorio para el idioma
		$language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

		// Determinar el código de idioma
		$this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

		// Cargar el archivo de idioma desde el directorio correcto
		$this->lang->load('aplicacion_lang', $language_directory_name_value);

		$this->load->database();
		$this->load->helper('url');

		$this->load->library('grocery_CRUD');

		$this->load->model('Config_model');
		$this->load->library('form_validation');

		$this->user = $this->User_model->getUser($this->session->userdata('email'));
		$this->menu = $this->User_model->getSideMenu($this->session->userdata('role_id'));
		$this->subMenu = $this->User_model->getSideSubMenu($this->menu);
		$this->load->model('Seo_model'); // Cargar el modelo SEO
		$this->data['seo'] = $this->Seo_model->get_seo_settings(); // Obtener configuraciones SEO
	}

	public function actualizar_ultima_actividad() {
		$email = $this->session->userdata('email'); // Obtener el email del usuario desde la sesión
		$this->User_model->updateLastActivity($email); // Actualizar la última actividad
	}

	public function _example_output($output = null, $title = 'Default Title') {
		$this->actualizar_ultima_actividad();
		$data['title'] = $title;
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		// Combinar el output de Grocery CRUD con los otros datos
		if ($output) {
			$data = array_merge($data, (array) $output);
		}

		$this->load->view('templates/header', $data);
		$this->load->view('templates/sidebar', $data);
		$this->load->view('templates/topbar', $data);
		$this->load->view('admin/dbplugins.php', $data);
		$this->load->view('templates/footerdb'); //para que no choque grocerycrud
	}

	public function index() {
		try {
			$crud = new grocery_CRUD();
// Unset jQuery to avoid loading it
			//$crud->unset_jquery();
			// Set the theme for the CRUD (datatables, flexigrid, bootstrap, etc.)
			$crud->set_theme('datatables');

			// Set the table for CRUD operations
			$crud->set_table('plugins'); // Set the table to 'plugins'

			// Set the subject of the CRUD (singular and plural forms)
			$crud->set_subject('Plugin', 'Plugins');

			// Specify which columns to display in the list view
			$crud->columns('id', 'name', 'version', 'description', 'author', 'created_at', 'installed');

			// Set the language for grocery CRUD
			$crud->set_language('english');

			// Set display names for columns
			$crud->display_as('name', 'Plugin Name');
			$crud->display_as('version', 'Version');
			$crud->display_as('description', 'Description');
			$crud->display_as('author', 'Author');
			$crud->display_as('created_at', 'Created At');
			$crud->display_as('installed', 'Installed');

			// Render the CRUD
			$output = $crud->render();

			// Pass the output and title to the example output function
			$this->_example_output($output, 'Plugin Database Manager');

		} catch (Exception $e) {
			show_error($e->getMessage() . ' --- ' . $e->getTraceAsString());
		}
	}

	public function blocked_ips() {
		try {
			$crud = new grocery_CRUD();

			// Set the theme for the CRUD
			$crud->set_theme('bootstrap-v4');

			// Set the table for CRUD operations
			$crud->set_table('blocked_ips'); // Set the table to 'blocked_ips'

			// Set the subject of the CRUD (singular and plural forms)
			$crud->set_subject('Blocked IP', 'Blocked IPs');

			// Specify which columns to display in the list view
			$crud->columns('id', 'ip_address', 'attempts', 'blocked_until', 'last_attempt');

			// Set display names for columns
			$crud->display_as('ip_address', 'IP Address');
			$crud->display_as('attempts', 'Attempts');
			$crud->display_as('blocked_until', 'Blocked Until');
			$crud->display_as('last_attempt', 'Last Attempt');

			// Set the language for grocery CRUD
			$crud->set_language('english');

			// Render the CRUD
			$output = $crud->render();

			// Pass the output and title to the example output function
			$this->_example_output($output, 'Blocked IP Database');

		} catch (Exception $e) {
			show_error($e->getMessage() . ' --- ' . $e->getTraceAsString());
		}
	}

}
