<?php
/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-09 16:54:52
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-11 14:40:48
 *
 * Adapted and updated from the code by https://github.com/sandhikagalih and
 * https://github.com/rafli-sss/wpu-login-pro, both under the MIT License.
 * All new code and additional functionalities created by Emo.
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */
defined('BASEPATH') or exit('No direct script access allowed');

class Menu extends CI_Controller {
	private $user, $menu, $subMenu;

	public function __construct() {

		parent::__construct();
		is_logged_in();

		// Obtener los datos del usuario desde la sesión
		$this->user = $this->session->userdata();

		//verificar es admin si no redirccionar
		if ($this->user['role_id'] != 1) {
			// Redirigir a la página por defecto del usuario
			redirect($this->session->userdata('redirection'));
		}

		// Verificar si los datos del usuario están disponibles
		if (empty($this->user)) {
			redirect('auth'); // Redirige al login si no hay datos de usuario
		}

		// Obtener el email del usuario
		$email = $this->user['email'];
		$this->load->model('User_model');
		// Obtener el idioma del usuario desde la base de datos

		$default_language = $this->User_model->get_user_language($email);

		$friendly_language = $default_language;

		// Mapeo de nombres amigables a códigos de idioma estándar
		$this->load->config('language');
		$language_directory = $this->config->item('language_directory');
		$language_directory_name = $this->config->item('language_directory_name');

		// Obtener el nombre del directorio para el idioma
		$language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

		// Determinar el código de idioma
		$this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

		// Cargar el archivo de idioma desde el directorio correcto
		$this->lang->load('aplicacion_lang', $language_directory_name_value);

		$this->load->model('Menu_model');
		$this->load->model('Config_model');
		$this->user = $this->User_model->getUser($this->session->userdata('email'));
		$this->menu = $this->User_model->getSideMenu($this->session->userdata('role_id'));
		$this->subMenu = $this->User_model->getSideSubMenu($this->menu);

		$this->load->model('Seo_model'); // Cargar el modelo SEO
		$this->data['seo'] = $this->Seo_model->get_seo_settings(); // Obtener configuraciones SEO
		// Verificar si la clase Form_validation ya está cargada
		if (!isset($this->form_validation)) {
			$this->load->library('form_validation');
		}

	}

	/**
	 * Menu index
	 */
	public function index() {
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$data['title'] = $this->lang->line('Menu Management');
		$data['menuManage'] = $this->Menu_model->getMenus();

		$this->form_validation->set_rules('menu', 'Menu', 'required');
		$this->form_validation->set_rules('icon', 'Icon', 'required');

		if (!$this->form_validation->run()) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('menu/index', $data);
			$this->load->view('templates/footer');
		} else {
			$data = [
				'menu' => $this->input->post('menu'),
				'icon' => $this->input->post('icon'),
			];
			$this->Menu_model->addMenu($data);
			$this->session->set_flashdata('alert-success', $this->lang->line('New menu has been added!'));
			redirect('menu');
		}
	}

	/**
	 * Menu Submenu
	 */
	public function submenu() {
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$data['title'] = $this->lang->line('Sub Menu Management');
		$data['menuManage'] = $this->Menu_model->getMenus();
		$data['subMenuManage'] = $this->Menu_model->getSubMenus();

		$this->form_validation->set_rules('title', 'Title', 'required');
		$this->form_validation->set_rules('menu_id', 'Menu', 'required');
		$this->form_validation->set_rules('url', 'URL', 'required');
		$this->form_validation->set_rules('icon', 'Icon', 'required');

		if (!$this->form_validation->run()) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('menu/submenu/index', $data);
			$this->load->view('templates/footer');
		} else {
			$data = [
				'title' => $this->input->post('title'),
				'menu_id' => $this->input->post('menu_id'),
				'url' => $this->input->post('url'),
				'icon' => $this->input->post('icon'),
				'is_active' => (is_null($this->input->post('is_active'))) ? 0 : 1,
			];
			$this->Menu_model->addSubMenu($data);
			$this->session->set_flashdata('alert-success', $this->lang->line('New sub menu has been added!'));
			redirect('menu/submenu');
		}
	}

	/**
	 * Menu Edit
	 */
	public function edit() {
		$type = $this->input->get('type');
		$id = $this->input->get('id');

		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$this->form_validation->set_rules('title', 'Title', 'required');
		$this->form_validation->set_rules('icon', 'Icon', 'required');

		if (!$this->form_validation->run()) {
			$data['title'] = 'Edit ' . ucfirst($type);

			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);

			if ($type == 'menu' && is_numeric($id)) {
				$data['editMenu'] = $this->Menu_model->getMenuById($id);
				$this->load->view('menu/edit', $data);
			} elseif ($type == 'submenu' && is_numeric($id)) {
				$data['editSub'] = $this->Menu_model->getSubMenuById($id); // Get 1 submenu by id
				$data['menuManage'] = $this->Menu_model->getMenus(); // Get semua menu
				$this->load->view('menu/submenu/edit', $data);
			} else {
				redirect('auth/blocked');
			}

			$this->load->view('templates/footer');
		} else {
			if ($type == 'menu') {
				$data = [
					'id' => $this->input->post('id'),
					'menu' => $this->input->post('title'),
					'icon' => $this->input->post('icon'),
				];
				$this->Menu_model->editMenu($data);
				$this->session->set_flashdata('alert-success', $this->lang->line('Menu changed successfully'));
				redirect('menu');
			} elseif ($type == 'submenu') {
				$post = $this->input->post();
				$data = [
					'id' => $post['id'],
					'menu_id' => $post['menu_id'],
					'title' => $post['title'],
					'url' => $post['url'],
					'icon' => $post['icon'],
					'is_active' => ($id <= 2) ? 1 : $post['is_active'],
					// Menu Dashboard dan My Profile harus selalu active
				];
				$this->Menu_model->editSubMenu($data);
				$this->session->set_flashdata('alert-success', $this->lang->line('Sub menu changed successfully'));
				redirect('menu/submenu');
			}
		}
	}

	/**
	 * Menu delete.
	 */
	public function delete() {
		$type = $this->input->get('type');
		$id = $this->input->get('id');

		if ($type == 'menu' && is_numeric($id)) {
			// Los menús de administrador, usuario y menú de la base de datos tienen el identificador 1,2,3 (base de datos) y el menú no debe eliminarse
			if ($id <= 3) {
				$this->session->set_flashdata('alert-danger', $this->lang->line('menu_cannot_delete'));
				redirect('menu');
			}
			$this->Menu_model->deleteMenu($id);
			$this->session->set_flashdata('alert-success', $this->lang->line('Menu has been deleted'));
			redirect('menu');
		} elseif ($type == 'sub' && is_numeric($id)) {
			// El submenú del panel, mi perfil, editar perfil, el menú de administración, el submenú de administración, el rol , cambiar contraseña , ajustes de la app y gestion de usuarios tienen el identificador 1,2,3,4,5,11,12,13,14 (base de datos) y estos submenús no deben eliminarse
			if ($id <= 5 || $id == 11 || $id == 12 || $id == 13 || $id == 14 || $id == 15 || $id == 16 || $id == 17 || $id == 18 || $id == 19) {
				$this->session->set_flashdata('alert-danger', $this->lang->line('submenu_cannot_delete'));
				redirect('menu/submenu');
			}
			$this->Menu_model->deleteSubMenu($id);
			$this->session->set_flashdata('alert-success', $this->lang->line('Sub menu has been deleted'));
			redirect('menu/submenu');
		} else {
			redirect('auth/blocked');
		}
	}
}
