<?php
/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-09 16:54:52
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-30 20:18:16
 *
 * Adapted and updated from the code by https://github.com/sandhikagalih and
 * https://github.com/rafli-sss/wpu-login-pro, both under the MIT License.
 * All new code and additional functionalities created by Emo.
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') or exit('No direct script access allowed');

class User extends CI_Controller {
	private $user, $menu, $subMenu;

	public function __construct() {

		parent::__construct();
		is_logged_in();

		// Obtener los datos del usuario desde la sesión
		$this->user = $this->session->userdata();

		// Verificar si los datos del usuario están disponibles
		if (empty($this->user)) {
			redirect('auth'); // Redirige al login si no hay datos de usuario
		}

		// Obtener el email del usuario
		$email = $this->user['email'];
		$this->load->model('User_model');

		// Obtener el idioma del usuario desde la base de datos

		$default_language = $this->User_model->get_user_language($email);

		$friendly_language = $default_language;

		// Mapeo de nombres amigables a códigos de idioma estándar
		$this->load->config('language');
		$language_directory = $this->config->item('language_directory');
		$language_directory_name = $this->config->item('language_directory_name');

		// Obtener el nombre del directorio para el idioma
		$language_directory_name_value = isset($language_directory_name[$friendly_language]) ? $language_directory_name[$friendly_language] : 'english';

		// Determinar el código de idioma
		$this->lang_code = isset($language_directory[$friendly_language]) ? $language_directory[$friendly_language] : 'en';

		// Cargar el archivo de idioma desde el directorio correcto
		$this->lang->load('aplicacion_lang', $language_directory_name_value);

		$this->load->model('Config_model');

		$this->load->helper('url');

		$this->load->library('grocery_CRUD');
		$this->load->library('table'); // Cargar la librería Table de CodeIgniter

		$this->user = $this->User_model->getUser($this->session->userdata('email'));
		$this->menu = $this->User_model->getSideMenu($this->session->userdata('role_id'));
		$this->subMenu = $this->User_model->getSideSubMenu($this->menu);
		$this->load->model('Seo_model'); // Cargar el modelo SEO
		$this->data['seo'] = $this->Seo_model->get_seo_settings(); // Obtener configuraciones SEO

		// Verificar si la clase Form_validation ya está cargada
		if (!isset($this->form_validation)) {
			$this->load->library('form_validation');
		}

	}

	public function actualizar_ultima_actividad() {
		$email = $this->session->userdata('email'); // Obtener el email del usuario desde la sesión

		// Llamar al método del modelo para actualizar la última actividad
		$this->User_model->updateLastActivity($email);
	}

	/**
	 * User index
	 */
	public function index() {
		$this->actualizar_ultima_actividad();

		$data['title'] = 'Inicio';
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma
		// Verificar el estado de la configuración 'activar-menu-lateral'
		$config_value = $this->Config_model->get_config_value('activar_menu_lateral');

		// Obtener el valor específico de 'activar_menu_lateral' del array devuelto
		$activar = isset($config_value['value']) ? intval($config_value['value']) : 0;

		// Depuración: mostrar el valor de $activar
		//var_dump($activar);

		// Verificar si el valor de $activar es igual a 1 para activar el sidebar
		if ($activar === 1) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('user/index', $data);
			$this->load->view('templates/footer');
		} else {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('user/profile', $data);
			$this->load->view('templates/footer');
		}
	}

	/**
	 * User profile
	 */
	public function profile() {
		$this->actualizar_ultima_actividad();
		$data['title'] = $this->lang->line('My_Profile');
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$this->load->view('templates/header', $data);
		$this->load->view('templates/sidebar', $data);
		$this->load->view('templates/topbar', $data);
		$this->load->view('user/profile', $data);
		$this->load->view('templates/footer');

	}

	/**
	 * User Edit profile
	 */
	public function edit() {
		$this->actualizar_ultima_actividad();
		$data['title'] = $this->lang->line('Edit_Profile');
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		// Obtener la configuración de idiomas
		$this->load->config('language');
		$data['language_directory'] = $this->config->item('language_directory');
		$data['language_directory_name'] = $this->config->item('language_directory_name');

		$this->form_validation->set_rules('name', $this->lang->line('Full_Name'), 'required|trim');
		$this->form_validation->set_rules('timezone', 'Timezone', 'required|trim');
		$this->form_validation->set_rules('language', $this->lang->line('Language:'), 'required|trim');

		if (!$this->form_validation->run()) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('user/edit', $data);
			$this->load->view('templates/footer');
		} else {
			$name = $this->input->post('name');
			$email = $this->input->post('email');
			$deletePic = $this->input->post('deletePic');
			$old_image = $this->user['image'];
			$timezone = $this->input->post('timezone'); // Obtener el timezone desde el formulario
			$language = $this->input->post('language'); // Obtener el idioma desde el formulario

			// Si se selecciona eliminar la imagen actual
			if (!is_null($deletePic)) {
				unlink(FCPATH . 'assets/img/profile/' . $old_image);
				$old_image = 'default.png';
			}

			// Verificar si hay una nueva imagen para subir
			$image = $_FILES['image']['name'];
			if ($image) {
				// Generar nombre de archivo aleatorio
				$this->load->helper('string');
				$config['file_name'] = random_string('alnum', 16);

				$config['allowed_types'] = 'gif|jpg|png';
				$config['max_size'] = '4096';
				$config['upload_path'] = './assets/img/profile/resize/';

				$this->load->library('upload', $config);

				if ($this->upload->do_upload('image')) {
					if ($old_image != 'default.png') {
						unlink(FCPATH . 'assets/img/profile/' . $old_image);
					}

					$new_image = $this->upload->data('file_name');

					// Configurar redimensionamiento de imagen
					$configImg['source_image'] = $config['upload_path'] . $new_image;
					$configImg['new_image'] = './assets/img/profile/' . $new_image;
					$configImg['quality'] = 100;
					$configImg['maintain_ratio'] = TRUE;
					$configImg['width'] = 250;
					$configImg['height'] = 250;

					$this->load->library('image_lib', $configImg);
					$this->image_lib->resize();

					if ($this->image_lib->resize()) {
						unlink(FCPATH . 'assets/img/profile/resize/' . $new_image);
					} else {
						echo $this->image_lib->display_errors();
						die();
					}
				} else {
					$this->session->set_flashdata('message', $this->upload->display_errors());
					redirect('user/edit');
					die();
				}
			}

			if (isset($new_image)) {
				// Si el usuario sube una nueva imagen
				$this->User_model->editProfile($name, $email, $new_image, $timezone, $language);
			} else {
				// Si el usuario no sube una nueva imagen (mantener la imagen anterior)
				$this->User_model->editProfile($name, $email, $old_image, $timezone, $language);
			}

			$this->session->set_flashdata('message', 'edited');
			redirect('user/profile');
		}
	}

	/**
	 * User change language
	 */
	public function change_language() {
		$language = $this->input->post('language');
		$email = $this->user['email']; // Obtener el email del usuario actual

		if ($language) {
			// Actualizar el idioma en la base de datos
			$this->User_model->update_user_language($email, $language);

			// Redirigir al usuario a la página de perfil o a la página deseada
			redirect('user/edit');
		} else {
			// Redirigir a la página de perfil en caso de error
			redirect('user/profile');
		}
	}

	/**
	 * User change password
	 */
	public function changePassword() {
		$this->actualizar_ultima_actividad();
		$data['title'] = $this->lang->line('Change_Password');
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$this->form_validation->set_rules('current_password', $this->lang->line('Current_Password'), 'required|trim');
		$this->form_validation->set_rules('new_password1', $this->lang->line('New_Password'), 'required|trim|min_length[8]|matches[new_password2]');
		$this->form_validation->set_rules('new_password2', $this->lang->line('Confirm_New_Password'), 'required|trim|min_length[8]|matches[new_password1]');

		if (!$this->form_validation->run()) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view('user/changepassword', $data);
			$this->load->view('templates/footer');
		} else {
			$currPw = $this->input->post('current_password');
			$newPw = $this->input->post('new_password1');

			if (!password_verify($currPw, $this->user['password'])) {
				$this->session->set_flashdata('alert-danger', $this->lang->line('Wrong_Current_Password!'));
				redirect('user/changePassword');
			} else {
				if ($currPw == $newPw) {
					$this->session->set_flashdata('alert-danger', $this->lang->line('The new password cannot be the same as the old password!'));
					redirect('user/changePassword');
				} else {
					// Password ok
					$hashPw = password_hash($newPw, PASSWORD_DEFAULT);
					$this->User_model->updatePassword($hashPw, $this->session->userdata('email'));

					$this->session->set_flashdata('alert-success', $this->lang->line('Password changed!'));
					redirect('user/changePassword');
				}
			}
		}
	}

	/**
	 * User delete account
	 */
	public function delete_account() {
		// Obtener los datos del usuario desde la sesión
		$user = $this->session->userdata();

		// Verificar si los datos del usuario están disponibles
		if (empty($user)) {
			redirect('auth'); // Redirige al login si no hay datos de usuario
		}

		$user_email = $user['email']; // Obtener el correo electrónico del usuario desde la sesión

		// Eliminar la cuenta del usuario
		$this->db->where('email', $user_email);
		$this->db->delete('user'); // Ajusta 'user' al nombre de tu tabla de usuarios

		// Verificar si el usuario fue eliminado
		if ($this->db->affected_rows() > 0) {
			// Destruir la sesión del usuario
			$this->session->sess_destroy();

			// Redirigir a la página de inicio o a la página de registro
			redirect('auth');
		} else {
			// Manejar el error si no se pudo eliminar el usuario
			$this->session->set_flashdata('message', 'Error al eliminar la cuenta.');
			redirect('user/profile'); // Ajusta 'user/profile' a la URL correcta de tu perfil
		}
	}

	/**
	 * Extenal page1
	 */
	public function page1() {
		$this->load_page('user/page1');
	}
	/**
	 * Extenal page2
	 */
	public function page2() {
		$this->load_page('user/page2');
	}

	/**
	 * Extenal page3
	 */
	public function page3() {
		$this->load_page('user/page3');
	}
	/**
	 * Extenal page4
	 */
	public function page4() {
		$this->load_page('user/page4');
	}
	/**
	 * Extenal page5
	 */
	public function page5() {
		$this->load_page('user/page5');
	}
	/**
	 * Extenal page6
	 */
	public function page6() {
		$this->load_page('user/page6');
	}
	/**
	 * Extenal page7
	 */
	public function page7() {
		$this->load_page('user/page7');
	}
	/**
	 * Extenal page8
	 */
	public function page8() {
		$this->load_page('user/page8');
	}
	/**
	 * Extenal page9
	 */
	public function page9() {
		$this->load_page('user/page9');
	}
	/**
	 * Extenal page10
	 */
	public function page10() {
		$this->load_page('user/page10');
	}

	/**
	 * Loads a external page.
	 *
	 * @param      <type>  $view   The view
	 */
	private function load_page($view) {
		$url = str_replace('user/', '', $view);
		$title = $this->User_model->get_title_by_url($url);

		$this->actualizar_ultima_actividad();
		$data['title'] = $title;
		$data['user'] = $this->user;
		$data['menu'] = $this->menu;
		$data['subMenu'] = $this->subMenu;
		$data['lang'] = $this->lang->language; // Pasar el array de idioma a la vista
		$data['lang_code'] = $this->lang_code; // Código del idioma

		$config_value = $this->Config_model->get_config_value('activar_menu_lateral');

		// Obtener el valor específico de 'activar_menu_lateral' del array devuelto
		$activar = isset($config_value['value']) ? intval($config_value['value']) : 0;

		if ($activar === 1) {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/sidebar', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view($view, $data);
			$this->load->view('templates/footer');
		} else {
			$this->load->view('templates/header', $data);
			$this->load->view('templates/topbar', $data);
			$this->load->view($view, $data);
			$this->load->view('templates/footer');
		}
	}

}
