<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-08-11 08:37:57
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-13 08:13:36
 * @website: https://loginbasicoemo.com
 *
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */
//Class to manage plugins
class HookManager {
	private $hooks = array();

	// Añadir un hook y asociarlo a una función de callback
	public function add_hook($hook_name, $callback) {
		if (is_callable($callback)) {
			if (!isset($this->hooks[$hook_name])) {
				$this->hooks[$hook_name] = array();
			}
			$this->hooks[$hook_name][] = $callback;
		}
	}

	public function load_plugins() {
		$plugin_dir = FCPATH . 'plugins/';
		$plugins = [];

		foreach (glob($plugin_dir . '*', GLOB_ONLYDIR) as $plugin_path) {
			$priority = 10; // Default priority
			$priority_file = $plugin_path . '/priority.txt';
			if (file_exists($priority_file)) {
				$priority = (int) file_get_contents($priority_file);
			}
			$plugins[] = ['path' => $plugin_path, 'priority' => $priority];
		}

		// Sort plugins by priority
		usort($plugins, function ($a, $b) {
			return $a['priority'] <=> $b['priority'];
		});

		// Load plugins in order
		foreach ($plugins as $plugin) {
			$init_file = $plugin['path'] . '/init.php';
			if (file_exists($init_file)) {
				require_once $init_file;
			}
		}
	}

	// Load a specific plugin by name
	public function load_specific_plugin($plugin_name) {
		$plugin_dir = FCPATH . 'plugins/' . $plugin_name;
		$init_file = $plugin_dir . '/init.php';

		if (file_exists($init_file)) {
			require_once $init_file;
			log_message('debug', "Plugin {$plugin_name} loaded successfully.");
		} else {
			log_message('error', "Plugin {$plugin_name} could not be found or does not have an init.php file.");
		}
	}

	public function run_hooks($hook_name) {
		$output = '';
		if (isset($this->hooks[$hook_name])) {
			foreach ($this->hooks[$hook_name] as $callback) {
				if (is_callable($callback)) {
					$output .= call_user_func($callback);
				}
			}
			log_message('debug', 'Hook ejecutado: ' . $hook_name);
		} else {
			log_message('debug', 'No hay hooks registrados para: ' . $hook_name);
		}
		return $output; // Retornar la salida acumulada
	}
}
