<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-11 14:28:41
 * @website: https://loginbasicoemo.com
 *
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') or exit('No direct script access allowed');

class Config_model extends CI_Model {

	/**
	 * Insert archive data
	 *
	 * @param      <type>  $data   The data
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function insert_archive($data) {
		return $this->db->insert('archivo', $data);
	}

	/**
	 * Gets the archived images.
	 *
	 * @return     <type>  The archived images.
	 */
	public function get_archived_images() {
		$this->db->select('id, titulo,descripcion, imagen,json');
		$query = $this->db->get('archivo');
		return $query->result_array();
	}

	/**
	 * Gets the archive by identifier.
	 *
	 * @param      <type>  $id     The identifier
	 *
	 * @return     <type>  The archive by identifier.
	 */
	public function get_archive_by_id($id) {
		$this->db->where('id', $id);
		$query = $this->db->get('archivo');
		return $query->row_array();
	}

	/**
	 * Gets all configuration.
	 *
	 * @return     <type>  All configuration.
	 */
	public function get_all_config() {
		$query = $this->db->get('config');
		return $query->result_array();
	}

	/**
	 * Sets the configuration value.
	 *
	 * @param      <type>  $key    The key
	 * @param      <type>  $value  The value
	 */
	public function set_config_value($key, $value) {
		$data = array(
			'value' => $value,
		);
		$this->db->where('key', $key);
		$this->db->update('config', $data);
	}

	/**
	 * Gets the configuration value.
	 *
	 * @param      <type>  $key    The key
	 *
	 * @return     <type>  The configuration value.
	 */
	public function get_config_value($key) {
		$this->db->where('key', $key);
		$query = $this->db->get('config');

		// Verificar si la consulta devolvió alguna fila
		if ($query->num_rows() > 0) {
			return $query->row_array();
		} else {
			return null;
		}
	}

	/**
	 * Gets the configuration value 2.
	 *
	 * @param      <type>  $key    The key
	 *
	 * @return     <type>  The configuration value 2.
	 */
	public function get_config_value2($key) {
		$this->db->where('key', $key);
		$query = $this->db->get('config');

		if ($query->num_rows() > 0) {
			$row = $query->row_array();
			return $row['value'];
		} else {
			return null;
		}
	}

	/**
	 * Update config
	 *
	 * @param      <type>  $key    The key
	 * @param      <type>  $value  The value
	 */
	public function update_config($key, $value) {
		$this->db->where('key', $key);
		$this->db->update('config', ['value' => $value]);
	}

	/**
	 * Update config value
	 *
	 * @param      <type>  $key    The key
	 * @param      <type>  $value  The value
	 */
	public function update_config_value($key, $value) {
		$data = array('value' => $value);
		$this->db->where('key', $key);
		$this->db->update('config', $data);
	}

	/**
	 * Determines if configuration table empty.
	 *
	 * @return     bool  True if configuration table empty, False otherwise.
	 */
	public function is_config_table_empty() {
		$query = $this->db->count_all('config');
		return $query == 0;
	}

	/**
	 * insert config value
	 *
	 * @param      <type>  $key    The key
	 * @param      <type>  $value  The value
	 */
	public function insert_config_value($key, $value) {
		$data = [
			'key' => $key,
			'value' => $value,
		];
		$this->db->insert('config', $data);
	}

	/**
	 * restore defauts if config table empty
	 */
	public function check_if_config_is_empty_and_restore_all_defaults() {
		$default_values = [

			'activar_registro' => '1',
			'activar_menu_lateral' => '1',
			'default_activation' => '1',

			'tipo_plantilla' => "normal",
			'font_family' => "Nunito",
			'custom_font' => '',
			'gradiente' => 'background:black;',
			'gradiente2' => 'background:blue;',
			'gradiente3' => 'background:black;',
			'page_background' => 'background:white;',
			'background_container' => 'background-color: white;padding:2rem;',
			'text_header' => 'color: #333333;',
			'button_color' => 'background-color: #007bff; color: #ffffff;border-radius:20px;',
			'email_input' => 'border-radius:20px',
			'password_input' => 'border-radius:20px',
			'checkbox_text' => 'color:gray;text-align: center;font-size: 12px;',
			'posicion' => 'display: flex; justify-content: center; align-items: center; height: 100vh;',
			'forgot_create' => 'color:#0066CC;text-align: center;',
			'animation_effect' => '',
			'texto_login' => $this->lang->line('texto_login'),
			'texto_email_placeholder' => $this->lang->line('texto_email_placeholder'),
			'texto_password_placeholder' => $this->lang->line('texto_password_placeholder'), // Asegúrate de definir esto en tu archivo de idioma
			'texto_terminos' => $this->lang->line('texto_terminos'),
			'texto_boton' => $this->lang->line('texto_boton'),
			'texto_forgot' => $this->lang->line('texto_forgot'),
			'texto_crear_cuenta' => $this->lang->line('texto_crear_cuenta'),
			'contenido_sidebar' => '<br><div style="color:white" ><span id="openSidebarBtn">&#9776; Menu</span><div class="container mt-5" style="color:white"><div class="container-fluid" style="height: 100%; display: flex; flex-direction: column; justify-content: center; align-items: center;"><div class="row w-100"><div class="col-12 text-center"><h1 class="display-4">Título del Producto</h1><p class="lead">Descripción breve del producto que puede ocupar una o dos líneas. Puede incluir características clave y beneficios.</p></div></div></div></div></div>',
			'contenido_terminos' => $this->lang->line('terms_conditions'),
			'css_placeholder' => '',
			'css_focus' => '',
			'css_general' => '',
			'source_image' => '',
			'footer' => '<footer class="sticky-footer bg-white">
    <div class="container my-auto">
    <div class="copyright text-center my-auto">
    <span id="copyright">Copyright &copy; emo ' . date("Y") . '</span>
    </div>
    </div>
    </footer>',
		];

		// Verificar si la tabla 'config' está vacía
		$is_empty = $this->Config_model->is_config_table_empty();

		if ($is_empty) {
			// La tabla está vacía, insertar todos los valores por defecto
			foreach ($default_values as $key => $new_value) {
				$this->Config_model->insert_config_value($key, $new_value);
			}
		}

	}

}
