<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-13 10:24:45
 * @website: https://loginbasicoemo.com
 *
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') or exit('No direct script access allowed');

class Menu_model extends CI_Model {

	/**
	 * Gets the menu.
	 *
	 * @param      <type>  $menu   The menu
	 *
	 * @return     <type>  The menu.
	 */
	public function getMenu($menu) {
		if (!empty($menu)) {
			$this->db->like('menu', $menu);
		}
		return $this->db->get('user_menu')->row_array();
	}

	/**
	 * Gets the menu by identifier.
	 *
	 * @param      <type>  $id     The identifier
	 *
	 * @return     <type>  The menu by identifier.
	 */
	public function getMenuById($id) {
		return $this->db->get_where('user_menu', ['id' => $id])->row_array();
	}

	/**
	 * Gets the menus.
	 *
	 * @return     <type>  The menus.
	 */
	public function getMenus() {
		return $this->db->get('user_menu')->result_array();
	}

	/**
	 * Adds a menu.
	 *
	 * @param      <type>  $data   The data
	 */
/*	public function addMenu($data) {
$this->db->insert('user_menu', ['menu' => $data]);
}
 */
	public function addMenu($data) {
		// This line should directly pass the $data array without re-wrapping it in another array.
		$this->db->insert('user_menu', $data);
	}

	/**
	 * Edit Menu
	 *
	 * @param      <type>  $data   The data
	 */
	public function editMenu($data) {
		$this->db->where('id', $data['id']);
		$this->db->update('user_menu', $data);
	}

	/**
	 * Delete Menu
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function deleteMenu($id) {
		$this->db->delete('user_menu', ['id' => $id]);
	}

	/**
	 * Gets the sub menus.
	 *
	 * @return     <type>  The sub menus.
	 */
	public function getSubMenus() {
		$query = "SELECT `user_sub_menu`.*, `user_menu`.`menu`
        FROM `user_sub_menu` JOIN `user_menu`
        ON `user_sub_menu`.`menu_id` = `user_menu`.`id`";
		return $this->db->query($query)->result_array();
	}

	/**
	 * Gets the sub menu is active.
	 *
	 * @param      <type>  $url    The url
	 *
	 * @return     array   The sub menu is active.
	 */
	public function getSubMenuIsActive($url) {
		$subMenu = $this->db->get_where('user_sub_menu', ['url' => $url]);

		if ($subMenu->num_rows() < 1) {
			return ['is_active' => 1];
		}

		return $subMenu->row_array()['is_active'];
	}

	/**
	 * Gets the sub menu by identifier.
	 *
	 * @param      <type>  $id     The identifier
	 *
	 * @return     <type>  The sub menu by identifier.
	 */
	public function getSubMenuById($id) {
		return $this->db->get_where('user_sub_menu', ['id' => $id])->row_array();
	}

	/**
	 * Adds a sub menu.
	 *
	 * @param      <type>  $data   The data
	 */
	public function addSubMenu($data) {
		$this->db->insert('user_sub_menu', $data);
	}

	/**
	 * Edit SubMenu
	 *
	 * @param      <type>  $data   The data
	 */
	public function editSubMenu($data) {
		$this->db->where('id', $data['id']);
		$this->db->update('user_sub_menu', $data);
	}

	/**
	 * Delete Sub Menu
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function deleteSubMenu($id) {
		$this->db->delete('user_sub_menu', ['id' => $id]);
	}

	/**
	 * User Access
	 *
	 * @param      <type>  $role_id  The role identifier
	 * @param      <type>  $menu_id  The menu identifier
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function userAccess($role_id, $menu_id) {
		return $this->db->get_where('user_access_menu', ['role_id' => $role_id, 'menu_id' => $menu_id]);
	}
}
