<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-11 14:35:42
 * @website: https://loginbasicoemo.com
 *
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

defined('BASEPATH') or exit('No direct script access allowed');

class User_model extends CI_Model {

	// Obtener datos del usuario por ID

	/**
	 * Gets the user.
	 *
	 * @param      <type>  $user_id  The user identifier
	 *
	 * @return     <type>  The user.
	 */
	public function get_user($user_id) {
		$query = $this->db->get_where('user', array('id' => $user_id));
		return $query->row_array();
	}

	// Actualizar el idioma del usuario

	/**
	 * Update language
	 *
	 * @param      <type>  $user_id   The user identifier
	 * @param      <type>  $language  The language
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function update_language($user_id, $language) {
		$this->db->where('id', $user_id);
		return $this->db->update('user', array('language' => $language));
	}

	// Método para obtener el idioma del usuario

	/**
	 * Gets the user language.
	 *
	 * @param      <type>  $email  The email
	 *
	 * @return     <type>  The user language.
	 */
	public function get_user_language($email) {
		$this->db->select('language');
		$this->db->from('user');
		$this->db->where('email', $email);
		$query = $this->db->get();

		if ($query->num_rows() > 0) {
			return $query->row()->language;
		} else {
			// Idioma por defecto si no se encuentra el usuario
			return $this->config->item('language');
		}
	}

	/**
	 * Update user language
	 *
	 * @param      <type>  $email     The email
	 * @param      <type>  $language  The language
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function update_user_language($email, $language) {
		$data = array('language' => $language);
		$this->db->where('email', $email);
		return $this->db->update('user', $data);
	}

	/**
	 * Gets the users.
	 *
	 * @return     <type>  The users.
	 */
	public function get_users() {
		// Obtener datos de los usuarios desde la base de datos
		$query = $this->db->get('user');

		return $query->result_array();
	}

	/**
	 * Gets the title by url.
	 *
	 * @param      <type>  $url    The url
	 *
	 * @return     string  The title by url.
	 */
	public function get_title_by_url($url) {
		$this->db->select('title');
		$this->db->from('user_sub_menu');
		$this->db->like('url', $url, 'both');
		$query = $this->db->get();

		if ($query->num_rows() > 0) {
			return $query->row()->title;
		} else {
			return '👤'; // Valor predeterminado en caso de que no se encuentre el título
		}
	}

	/**
	 * Edit profile
	 *
	 * @param      <type>  $name      The name
	 * @param      <type>  $email     The email
	 * @param      <type>  $image     The image
	 * @param      <type>  $timezone  The timezone
	 * @param      <type>  $language  The language
	 */
	public function editProfile($name, $email, $image, $timezone, $language) {
		$data = array(
			'name' => $name,
			'image' => $image,
			'timezone' => $timezone,
			'language' => $language,
		);

		$this->db->where('email', $email);
		$this->db->update('user', $data);
	}

	// Actualizar última actividad

	/**
	 * Update las activity
	 *
	 * @param      <type>  $email  The email
	 */
	public function updateLastActivity($email) {
		$this->db->where('email', $email);
		$this->db->update('user', array('ultima_actividad' => date('Y-m-d H:i:s')));
	}

	// Obtener usuarios en línea por hora

	/**
	 * Gets the users online by hour.
	 *
	 * @return     <type>  The users online by hour.
	 */
	public function get_users_online_by_hour() {
		// Obtener la zona horaria desde la configuración
		$timezone = $this->config->item('time_reference');
		if (!$timezone) {
			$timezone = '+00:00'; // UTC por defecto
		}

		// Obtener la fecha actual en formato 'YYYY-MM-DD'
		$today = date('Y-m-d');

		// Consulta SQL
		$sql = "SELECT HOUR(CONVERT_TZ(ultima_actividad, '+00:00', '{$timezone}')) as hora,
        COUNT(*) as cantidad_usuarios
        FROM user
        WHERE DATE(CONVERT_TZ(ultima_actividad, '+00:00', '{$timezone}')) = '{$today}'
        GROUP BY HOUR(CONVERT_TZ(ultima_actividad, '+00:00', '{$timezone}'))";

		// Ejecutar la consulta
		$query = $this->db->query($sql);

		// Obtener los resultados
		$result = $query->result_array();

		// Verificar si se devolvieron resultados
		if (empty($result)) {
			log_message('error', 'No data returned from the query');
		}

		return $result;
	}

	// Obtener total de usuarios en línea

	/**
	 * Gets the total online users.
	 *
	 * @return     <type>  The total online users.
	 */
	public function get_total_online_users() {
		// Definir el límite de tiempo para considerar a un usuario como "en línea"
		$online_threshold = strtotime('-15 minutes'); // Consideramos "en línea" si se ha accedido en los últimos 15 minutos

		// Consulta para contar usuarios con último acceso reciente
		$this->db->where('ultima_actividad >=', date('Y-m-d H:i:s', $online_threshold));
		$query = $this->db->get('user');

		return $query->num_rows();
	}

	// Actualizar último acceso

	/**
	 * Update last access
	 *
	 * @param      <type>  $email             The email
	 * @param      <type>  $ip                { parameter_description }
	 * @param      <type>  $tipo_dispositivo  The tipo dispositivo
	 * @param      <type>  $sistema           The sistema
	 * @param      <type>  $browser           The browser
	 */
	public function updateLastAccess($email, $ip, $tipo_dispositivo, $sistema, $browser) {
		$this->db->set('ultimo_acceso', date('Y-m-d H:i:s'));
		$this->db->set('ip', $ip);
		$this->db->set('tipo_dispositivo', $tipo_dispositivo);
		$this->db->set('sistema', $sistema);
		$this->db->set('browser', $browser);
		$this->db->set('session_count', 'session_count + 1', FALSE); // Incrementar el conteo de sesiones
		$this->db->where('email', $email);
		$this->db->update('user');
	}

	/**
	 * Adds an user.
	 *
	 * @param      <type>  $data   The data
	 */
	public function addUser($data) {
		$this->db->insert('user', $data);
	}

	public function deleteUser($email) {
		$this->db->delete('user', ['email' => $email]);
	}

	/**
	 * update user
	 *
	 * @param      <type>  $email  The email
	 */
	public function updateUser($email) {
		$this->db->set('is_active', 1);
		$this->db->where('email', $email);
		$this->db->update('user');
	}

	/**
	 * Gets the user.
	 *
	 * @param      <type>  $email  The email
	 * @param      string  $table  The table
	 *
	 * @return     <type>  The user.
	 */
	public function getUser($email, $table = 'user') {
		return $this->db->get_where($table, ['email' => $email])->row_array();
	}

	/**
	 * Gets the active user.
	 *
	 * @param      <type>  $email  The email
	 *
	 * @return     <type>  The active user.
	 */
	public function getActiveUser($email) {
		return $this->db->get_where('user', ['email' => $email, 'is_active' => 1])->row_array();
	}

	/**
	 * Gets the side menu.
	 *
	 * @param      <type>  $role_id  The role identifier
	 *
	 * @return     <type>  The side menu.
	 */
	public function getSideMenu($role_id) {
		$queryMenu = "SELECT `user_menu`.`id`, `user_menu`.`menu`, `user_menu`.`icon`
                  FROM `user_menu`
                  JOIN `user_access_menu` ON `user_menu`.`id` = `user_access_menu`.`menu_id`
                  WHERE `user_access_menu`.`role_id` = $role_id
                  ORDER BY `user_access_menu`.`menu_id` ASC";
		return $this->db->query($queryMenu)->result_array();
	}

	/**
	 * Gets the side sub menu.
	 *
	 * @param      <type>  $menu   The menu
	 *
	 * @return     array   The side sub menu.
	 */
	public function getSideSubMenu($menu) {
		$subMenu = [];
		foreach ($menu as $m) {
			$menuId = $m['id'];
			$querySubMenu = "SELECT *
            FROM `user_sub_menu`
            WHERE `menu_id` = $menuId
            AND `is_active` = 1";

			array_push($subMenu, $this->db->query($querySubMenu)->result_array());
		}
		return $subMenu;
	}

	/**
	 * Gets the roles.
	 *
	 * @return     <type>  The roles.
	 */
	public function getRoles() {
		return $this->db->get('user_role')->result_array();
	}

	/**
	 * Gets the role.
	 *
	 * @param      <type>  $id     The identifier
	 *
	 * @return     <type>  The role.
	 */
	public function getRole($id) {
		return $this->db->get_where('user_role', ['id' => $id])->row_array();
	}

	/**
	 * Adds a role.
	 *
	 * @param      <type>  $data   The data
	 */
	public function addRole($data) {
		$this->db->insert('user_role', $data);
	}

	/**
	 * Edit Role
	 *
	 * @param      <type>  $id     The identifier
	 * @param      <type>  $data   The data
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function editRole($id, $data) {
		$this->db->where('id', $id);
		return $this->db->update('user_role', $data); // Corrige el nombre de la tabla
	}

	/**
	 * Delete Role
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function deleteRole($id) {
		$this->db->delete('user_role', ['id' => $id]);
	}

	/**
	 * Access Menu
	 *
	 * @param      <type>  $role_id  The role identifier
	 * @param      <type>  $menu_id  The menu identifier
	 *
	 * @return     <type>  ( description_of_the_return_value )
	 */
	public function accessMenu($role_id, $menu_id) {
		$this->db->where('role_id', $role_id);
		$this->db->where('menu_id', $menu_id);
		return $this->db->get('user_access_menu');
	}

	/**
	 * Gets the access.
	 *
	 * @param      <type>  $data   The data
	 *
	 * @return     <type>  The access.
	 */
	public function getAccess($data) {
		return $this->db->get_where('user_access_menu', $data);
	}

	/**
	 * Adds an access.
	 *
	 * @param      <type>  $data   The data
	 */
	public function addAccess($data) {
		$this->db->insert('user_access_menu', $data);
	}

	/**
	 * Delete access
	 *
	 * @param      <type>  $data   The data
	 */
	public function deleteAccess($data) {
		$this->db->delete('user_access_menu', $data);
	}

	/**
	 * Update password
	 *
	 * @param      <type>  $pw     The password
	 * @param      <type>  $email  The email
	 */
	public function updatePassword($pw, $email) {
		$this->db->set('password', $pw);
		$this->db->where('email', $email);
		$this->db->update('user');
	}

	/**
	 * insert token
	 *
	 * @param      <type>  $data   The data
	 */
	public function insertToken($data) {
		$this->db->insert('user_token', $data);
	}

	/**
	 * Gets the user by token.
	 *
	 * @param      <type>  $token  The token
	 *
	 * @return     <type>  The user by token.
	 */
	public function getUserByToken($token) {
		return $this->db->get_where('user_token', ['token' => $token])->row_array();
	}

	/**
	 * Delete token
	 *
	 * @param      <type>  $email  The email
	 */
	public function deleteToken($email) {
		$this->db->delete('user_token', ['email' => $email]);
	}

	/**
	 * Gets the number of users.
	 *
	 * @return     string  The number of users.
	 */
	public function getNumberOfUsers() {
		$query = "SELECT COUNT(*) as num_users FROM `user`";
		$resultado = $this->db->query($query)->row();
		if ($resultado) {
			return $resultado->num_users;
		} else {
			return '0'; // En caso de que no haya usuarios, devolvemos '0' como string
		}
	}

	/**
	 * Gets the number of active users.
	 *
	 * @return     string  The number of active users.
	 */
	public function getNumberOfActiveUsers() {
		$query = "SELECT COUNT(*) as num_users FROM `user` where `is_active`=1";
		$resultado = $this->db->query($query)->row();
		if ($resultado) {
			return $resultado->num_users;
		} else {
			return '0'; // En caso de que no haya usuarios, devolvemos '0' como string
		}
	}

}
