<!DOCTYPE html>
<html>
<head>
    <title><?php echo $this->lang->line('edit_pwa_files'); ?></title>

    <script src="<?=base_url('assets/vendor/ace/ace.js');?>"></script>
    <script src="<?=base_url('assets/vendor/ace/mode-json.js');?>"></script>
    <script src="<?=base_url('assets/vendor/ace/mode-javascript.js');?>"></script>
    <script src="<?=base_url('assets/vendor/ace/theme-monokai.js');?>"></script>

    <style>
        .editor {
            width: 100%;
            height: 400px;
        }
        .editor.fullscreen {
            position: fixed;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            z-index: 9999;
        }
        @media (max-width: 767.98px) {
            .editor-container {
                margin-bottom: 20px;
            }
        }
        .btn-icon {
            display: inline-flex;
            align-items: center;
            justify-content: center;
            padding: 10px;
            border-radius: 5px;
            border: none;
            cursor: pointer;
            transition: background-color 0.3s;
        }
        .btn-icon i {
            font-size: 20px;
        }
        .btn-primary {
            background-color: #007bff;
            color: white;
        }
        .btn-primary:hover {
            background-color: #0056b3;
        }
        .btn-secondary {
            background-color: #6c757d;
            color: white;
        }
        .btn-secondary:hover {
            background-color: #5a6268;
        }
        .fullscreen-button {
            position: absolute;
            top: 10px;
            right: 10px;
            z-index: 1000;
        }
    </style>
</head>
<body>
    <div class="container-fluid">
        <div class="row">
            <div class="col-lg-12">
                <div class="card border-0 shadow-sm h-100">
                    <div class="card-body">
                        <h2 class="card-title text-primary"><i class="fas fa-edit"></i> <?php echo $this->lang->line('edit_pwa_files'); ?></h2>
                        <form method="post" action="<?=site_url('admin/save_pwa');?>">
                            <!-- Campo oculto para el token CSRF -->
                            <input type="hidden" name="<?=$this->security->get_csrf_token_name();?>" value="<?=$this->security->get_csrf_hash();?>" />

                            <div class="row">
                                <div class="col-md-6 editor-container mb-4 position-relative">
                                    <div class="form-group">
                                        <label for="manifest_editor"><?php echo $this->lang->line('manifest_json'); ?></label>
                                        <div id="manifest_editor" class="editor"></div>
                                        <textarea id="manifest_content" name="manifest_content" style="display:none;"><?=htmlspecialchars($manifest_content, ENT_QUOTES, 'UTF-8');?></textarea>
                                        <button type="button" class="btn btn-secondary btn-icon fullscreen-button" onclick="toggleFullscreen('manifest_editor')"><i class="fas fa-expand"></i></button>
                                    </div>
                                </div>
                                <div class="col-md-6 editor-container mb-4 position-relative">
                                    <div class="form-group">
                                        <label for="service_worker_editor"><?php echo $this->lang->line('service_worker_js'); ?></label>
                                        <div id="service_worker_editor" class="editor"></div>
                                        <textarea id="service_worker_content" name="service_worker_content" style="display:none;"><?=htmlspecialchars($service_worker_content, ENT_QUOTES, 'UTF-8');?></textarea>
                                        <button type="button" class="btn btn-secondary btn-icon fullscreen-button" onclick="toggleFullscreen('service_worker_editor')"><i class="fas fa-expand"></i></button>
                                    </div>
                                </div>
                            </div>

                            <button type="submit" class="btn btn-primary"><?php echo $this->lang->line('save'); ?></button>
                        </form>
                    </div>
                </div>
            </div>
        </div>

        <!-- Nuevo Contenedor -->
        <div class="row mt-4">
            <div class="col-lg-12">
                <div class="card border-0 shadow-sm h-100">
                    <div class="card-body">
                        <form method="post" action="<?=site_url('admin/save_pwa_script');?>">
                            <!-- Campo oculto para el token CSRF -->
                            <input type="hidden" name="<?=$this->security->get_csrf_token_name();?>" value="<?=$this->security->get_csrf_hash();?>" />

                            <div class="editor-container mb-4 position-relative">
                                <div class="form-group">
                                    <label for="pwa_script_editor">auth_header.php script</label>
                                    <div id="pwa_script_editor" class="editor"></div>
                                    <textarea id="pwa_script_content" name="pwa_script_content" style="display:none;"></textarea>
                                    <button type="button" class="btn btn-secondary btn-icon fullscreen-button" onclick="toggleFullscreen('pwa_script_editor')"><i class="fas fa-expand"></i></button>
                                </div>
                            </div>

                            <button type="submit" class="btn btn-primary"><?php echo $this->lang->line('save'); ?></button>
                            <button type="button" id="restoreDefaultButton" class="btn btn-secondary btn-icon"><i class="fas fa-undo"></i></button>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    </div>

<script>
    document.addEventListener("DOMContentLoaded", function() {
        var manifestContent = <?=json_encode($manifest_content);?>;
        var serviceWorkerContent = <?=json_encode($service_worker_content);?>;
        var scriptContent = <?=json_encode($script_content);?>;

        // Código por defecto
        var defaultScriptContent = `
if ('serviceWorker' in navigator) {
    window.addEventListener('load', () => {
        navigator.serviceWorker.register('<?php echo base_url('assets/service-worker.js'); ?>')
            .then((registration) => {
                console.log('Service Worker registered with scope:', registration.scope);
            }).catch((error) => {
                console.log('Service Worker registration failed:', error);
            });
    });
}

let deferredPrompt;

window.addEventListener('beforeinstallprompt', (e) => {
    e.preventDefault();
    deferredPrompt = e;
    showInstallPromotion(); // Función personalizada para mostrar el botón o promoción de instalación
});

function showInstallPromotion() {
    const installButton = document.createElement('div');
    installButton.innerHTML = \`
        <button class="fab">
            <i class="fas fa-download"></i>
        </button>
    \`;
    installButton.className = 'install-prompt';

    // Style the button
    const style = document.createElement('style');
    style.innerHTML = \`
        .install-prompt {
            position: fixed;
            bottom: 70px;
            right: 15px;
            z-index: 1000;
        }
        .fab {
            background-color: #76C2AF;
            color: white;
            border: none;
            border-radius: 50%;
            padding: 15px;
            box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
            cursor: pointer;
            transition: background-color 0.3s;
        }
        .fab:hover {
            background-color: #45a049;
        }
        .fab i {
            font-size: 24px;
        }
    \`;
    document.head.appendChild(style);

    installButton.addEventListener('click', () => {
        installButton.style.display = 'none';
        deferredPrompt.prompt();
        deferredPrompt.userChoice.then((choiceResult) => {
            if (choiceResult.outcome === 'accepted') {
                console.log('User accepted the A2HS prompt');
            } else {
                console.log('User dismissed the A2HS prompt');
            }
            deferredPrompt = null;
        });
    });

    document.body.appendChild(installButton);
}`;

        var manifestEditor = ace.edit("manifest_editor");
        manifestEditor.setTheme("ace/theme/monokai");
        manifestEditor.session.setMode("ace/mode/json");
        manifestEditor.setValue(manifestContent, -1);

        var serviceWorkerEditor = ace.edit("service_worker_editor");
        serviceWorkerEditor.setTheme("ace/theme/monokai");
        serviceWorkerEditor.session.setMode("ace/mode/javascript");
        serviceWorkerEditor.setValue(serviceWorkerContent, -1);

        var pwaScriptEditor = ace.edit("pwa_script_editor");
        pwaScriptEditor.setTheme("ace/theme/monokai");
        pwaScriptEditor.session.setMode("ace/mode/javascript");
        pwaScriptEditor.setValue(scriptContent, -1);

        // Actualizar contenido cuando se sale del modo de pantalla completa
        function updateContentOnExitFullscreen() {
            document.getElementById('manifest_content').value = manifestEditor.getValue();
            document.getElementById('service_worker_content').value = serviceWorkerEditor.getValue();
            document.getElementById('pwa_script_content').value = pwaScriptEditor.getValue();
        }

        document.addEventListener('fullscreenchange', function () {
            if (!document.fullscreenElement) {
                var editors = document.querySelectorAll('.editor.fullscreen');
                editors.forEach(function(editor) {
                    editor.classList.remove('fullscreen');
                });
                updateContentOnExitFullscreen();
            }
        });

        document.addEventListener('mozfullscreenchange', function () {
            if (!document.mozFullScreenElement) {
                var editors = document.querySelectorAll('.editor.fullscreen');
                editors.forEach(function(editor) {
                    editor.classList.remove('fullscreen');
                });
                updateContentOnExitFullscreen();
            }
        });

        document.addEventListener('webkitfullscreenchange', function () {
            if (!document.webkitFullscreenElement) {
                var editors = document.querySelectorAll('.editor.fullscreen');
                editors.forEach(function(editor) {
                    editor.classList.remove('fullscreen');
                });
                updateContentOnExitFullscreen();
            }
        });

        document.addEventListener('msfullscreenchange', function () {
            if (!document.msFullscreenElement) {
                var editors = document.querySelectorAll('.editor.fullscreen');
                editors.forEach(function(editor) {
                    editor.classList.remove('fullscreen');
                });
                updateContentOnExitFullscreen();
            }
        });

        // Al enviar el formulario, actualiza el contenido de los textarea ocultos con el valor del editor
        document.querySelector('form[action="<?=site_url('admin/save_pwa');?>"]').addEventListener('submit', function() {
            document.getElementById('manifest_content').value = manifestEditor.getValue();
            document.getElementById('service_worker_content').value = serviceWorkerEditor.getValue();
        });

        document.querySelector('form[action="<?=site_url('admin/save_pwa_script');?>"]').addEventListener('submit', function() {
            document.getElementById('pwa_script_content').value = pwaScriptEditor.getValue();
        });

        // Funcionalidad del botón "Restore Default"
        document.getElementById('restoreDefaultButton').addEventListener('click', function() {
            pwaScriptEditor.setValue(defaultScriptContent, -1);
        });

        // Funcionalidad del botón "Fullscreen"
        window.toggleFullscreen = function(editorId) {
            var editorElement = document.getElementById(editorId);
            var fullscreenButton = editorElement.nextElementSibling;

            if (!document.fullscreenElement) {
                editorElement.classList.add('fullscreen');
                fullscreenButton.innerHTML = '<i class="fas fa-compress"></i>';
                editorElement.requestFullscreen().catch(err => {
                    alert(`Error attempting to enable full-screen mode: ${err.message} (${err.name})`);
                });
            } else {
                document.exitFullscreen().then(() => {
                    editorElement.classList.remove('fullscreen');
                    fullscreenButton.innerHTML = '<i class="fas fa-expand"></i>';
                });
            }
        };

        // Mostrar mensaje de SweetAlert2
        <?php if ($message = $this->session->flashdata('message')): ?>
            Swal.fire({
                icon: '<?=$message === 'success' ? 'success' : 'error'?>',
                title: '<?=$message === 'success' ? $this->lang->line('saved_successfully') : $this->lang->line('error_saving_files')?>',
                text: '<?=$message === 'success' ? $this->lang->line('your_changes_saved') : $this->lang->line('problem_saving_changes')?>'
            });
        <?php endif;?>
    });
</script>

</body>
</html>
