<div class="container mt-5">
    <?php if ($this->session->flashdata('success')): ?>
        <div class="alert alert-success">
            <i class="fas fa-check-circle"></i> <?=$this->session->flashdata('success');?>
        </div>
    <?php endif;?>

    <?php if ($this->session->flashdata('error')): ?>
        <div class="alert alert-danger">
            <i class="fas fa-exclamation-circle"></i> <?=$this->session->flashdata('error');?>
        </div>
    <?php endif;?>

    <div class="row">
        <div class="col-md-12">
            <h1 class="text-center"><i class="fas fa-puzzle-piece"></i> <?=$this->lang->line('plugins_available');?></h1>
            <div class="text-right mb-3">
                <button id="refreshPluginsBtn" class="btn btn-primary">
                    <i class="fas fa-sync-alt"></i>
                </button>
            </div>
            <div class="table-responsive">
                <table class="table table-striped table-bordered mt-4">
                    <thead class="thead-dark">
                        <tr>
                            <th scope="col"><i class="fas fa-file-alt"></i> <?=$this->lang->line('plugin_name');?></th>
                            <th scope="col"><i class="fas fa-code-branch"></i> <?=$this->lang->line('plugin_version');?></th>
                            <th scope="col"><i class="fas fa-align-left"></i> <?=$this->lang->line('plugin_description');?></th>
                            <th scope="col"><i class="fas fa-user"></i> <?=$this->lang->line('plugin_author');?></th>
                            <th scope="col"><i class="fas fa-tools"></i> <?=$this->lang->line('plugin_action');?></th>
                        </tr>
                    </thead>
                    <tbody id="plugin-list">
                        <?php if (isset($plugins) && !empty($plugins)): ?>
                            <?php foreach ($plugins as $plugin): ?>
                            <tr>
                                <td><?=$plugin->name?></td>
                                <td>
                                    <?=$plugin->version?>
                                    <?php if ($plugin->available_version && version_compare($plugin->available_version, $plugin->version, '>')): ?>
                                        <br><small class="text-warning">New version available: <?=$plugin->available_version?></small>
                                    <?php endif;?>
                                </td>
                                <td><?=$plugin->description?></td>
                                <td><?=$plugin->author?></td>
                                <td class="text-center">
                                    <?php if ($plugin->installed): ?>
                                        <?php if ($plugin->available_version && version_compare($plugin->available_version, $plugin->version, '>')): ?>
                                            <a href="<?=site_url('admin/update_plugin/' . $plugin->id)?>" class="btn btn-warning btn-sm">
                                                <i class="fas fa-sync-alt"></i> <?=$this->lang->line('plugin_update');?>
                                            </a>
                                        <?php endif;?>
                                        <a href="<?=site_url('admin/uninstall_plugin/' . $plugin->id)?>" class="btn btn-danger btn-sm">
                                            <i class="fas fa-times-circle"></i> <?=$this->lang->line('plugin_uninstall');?>
                                        </a>
                                    <?php else: ?>
                                        <a href="<?=site_url('admin/install_plugin/' . $plugin->id)?>" class="btn btn-success btn-sm">
                                            <i class="fas fa-download"></i> <?=$this->lang->line('plugin_install');?>
                                        </a>
                                        <a href="<?=site_url('admin/download_plugin/' . $plugin->id)?>" class="btn btn-primary btn-sm">
                                            <i class="fas fa-puzzle-piece"></i> <?=$this->lang->line('plugin_download');?>
                                        </a>
                                    <?php endif;?>
                                </td>
                            </tr>
                            <?php endforeach;?>
                        <?php else: ?>
                            <!-- Aquí se llenará con JavaScript en el cliente -->
                        <?php endif;?>
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</div>
<script>
const baseUrl = '<?=base_url();?>';

function loadPlugins() {
    // Verificar si el dominio actual es diferente a loginbasicoemo.com
    if (window.location.hostname !== 'loginbasicoemo.com') {
        fetch('https://loginbasicoemo.com/api/plugins')
            .then(response => response.json())
            .then(data => {
                // Enviar los datos al servidor del cliente para que los guarde en la base de datos
                return fetch(`${baseUrl}api/save_plugins`, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify(data) // Enviar los plugins como JSON
                });
            })
            .then(response => response.json())
            .then(result => {
                if (result.status === 'success') {
                    console.log('Plugins saved successfully');
                    // Recargar la página después de guardar los plugins con éxito
                    window.location.reload();
                } else {
                    console.error('Failed to save plugins:', result.message);
                }
            })
            .catch(error => console.error('Error processing plugins:', error));
    }
}

document.addEventListener('DOMContentLoaded', function() {
    <?php if (!isset($plugins) || empty($plugins)): ?>
        loadPlugins();
    <?php endif;?>

    document.getElementById('refreshPluginsBtn').addEventListener('click', function() {
        loadPlugins();
    });
});
</script>
