const CACHE_VERSION = 'v2'; // Cambia este valor para versionar el caché
const CACHE_NAME = `loginbasicoEmo-cache-${CACHE_VERSION}`;

// Lista de URLs a cachear en la instalación
const urlsToCache = [
  '/loginbasicoemopro/', // Página principal de la app
  '/loginbasicoemopro/index.php', 
  '/loginbasicoemopro/assets/img/icon-192x192.png',
  '/loginbasicoemopro/assets/img/icon-512x512.png',

];

// Instalación del Service Worker
self.addEventListener('install', (event) => {
  event.waitUntil(
    caches.open(CACHE_NAME)
      .then((cache) => {
        console.log('[Service Worker] Caching new resources');
        return cache.addAll(urlsToCache);
      })
      .catch((error) => {
        console.error('[Service Worker] Error al cachear los recursos:', error);
      })
  );
});

// Activación del Service Worker
self.addEventListener('activate', (event) => {
  const cacheWhitelist = [CACHE_NAME];
  
  event.waitUntil(
    caches.keys().then((cacheNames) => {
      return Promise.all(
        cacheNames.map((cacheName) => {
          if (!cacheWhitelist.includes(cacheName)) {
            console.log('[Service Worker] Deleting old cache:', cacheName);
            return caches.delete(cacheName);
          }
        })
      );
    })
  );
});

// Recuperación de Recursos
self.addEventListener('fetch', (event) => {
  event.respondWith(
    caches.match(event.request)
      .then((cachedResponse) => {
        if (cachedResponse) {
          return cachedResponse;
        }
        return fetch(event.request)
          .then((response) => {
            const responseClone = response.clone();
            caches.open(CACHE_NAME)
              .then((cache) => {
                cache.put(event.request, responseClone);
              });
            return response;
          })
          .catch(() => {
            // Si la solicitud falla, intenta servir la página principal cacheada
            return caches.match('/');
          });
      })
  );
});
