<?php
class Database {

	// Función para probar la conexión a la base de datos
	public function test_connection($config) {
		$conn = new mysqli($config['hostname'], $config['username'], $config['password']);
		if ($conn->connect_error) {
			return $conn->connect_error; // Devolver el mensaje de error de conexión
		}
		$conn->close();
		return true; // Conexión exitosa
	}

	// Función para crear la base de datos
	public function create_database($data) {
		$mysqli = new mysqli($data['hostname'], $data['username'], $data['password']);
		if ($mysqli->connect_error) {
			return $mysqli->connect_error; // Devolver el mensaje de error de conexión
		}

		// Intentar crear la base de datos
		if (!$mysqli->query("CREATE DATABASE IF NOT EXISTS " . $mysqli->real_escape_string($data['database']))) {
			$mysqli->close();
			return $mysqli->error; // Devolver el mensaje de error si la consulta falla
		}

		$mysqli->close();
		return true; // Base de datos creada exitosamente
	}

	// Función para crear las tablas
	public function create_tables($data) {
		$mysqli = new mysqli($data['hostname'], $data['username'], $data['password'], $data['database']);
		if ($mysqli->connect_error) {
			return $mysqli->connect_error; // Devolver el mensaje de error de conexión
		}

		// Leer el archivo SQL para crear tablas
		$query = file_get_contents('assets/database.sql');
		if ($query === false) {
			$mysqli->close();
			return "Error reading SQL file."; // Error al leer el archivo SQL
		}

		if (!$mysqli->multi_query($query)) {
			$mysqli->close();
			return $mysqli->error; // Devolver el mensaje de error si la consulta falla
		}

		// Esperar a que todas las consultas se completen
		while ($mysqli->more_results() && $mysqli->next_result()) {
			// Si hay un error en cualquier consulta, se detecta aquí
			if ($mysqli->error) {
				$mysqli->close();
				return $mysqli->error; // Devolver el mensaje de error
			}
		}

		$mysqli->close();
		return true; // Tablas creadas exitosamente
	}

	public function update_user_language($data) {
		$mysqli = new mysqli($data['hostname'], $data['username'], $data['password'], $data['database']);

		if ($mysqli->connect_error) {
			return "Connection failed: " . $mysqli->connect_error;
		}

		$language = $mysqli->real_escape_string($data['language']);

		// Actualizar el idioma de todos los usuarios
		$update_users_query = "UPDATE user SET language = '$language'";
		if ($mysqli->query($update_users_query) !== TRUE) {
			$mysqli->close();
			return "Error updating user records: " . $mysqli->error;
		}

		// Establecer el valor predeterminado del idioma en la estructura de la base de datos
		$alter_table_query = "ALTER TABLE user ALTER COLUMN language SET DEFAULT '$language'";
		if ($mysqli->query($alter_table_query) !== TRUE) {
			$mysqli->close();
			return "Error setting default language: " . $mysqli->error;
		}

		$mysqli->close();
		return true;
	}

}
?>
