<?php

class Useful_tool_links {

	private $lang = [];

	// Load the appropriate language file
	public function load_language($language) {
		$language_file = __DIR__ . "/languages/{$language}/plugin_lang.php";
		if (file_exists($language_file)) {
			include $language_file;
			$this->lang = $lang;
			log_message('debug', "Language {$language} loaded successfully from {$language_file}.");
		} else {
			log_message('error', "Failed to load language file: {$language_file}");
		}
	}

	// Register hooks for the plugin
	public function register_hooks($hook_manager) {
		// Register the plugin to the 'ajustes_tools' hook
		$hook_manager->add_hook('ajustes_tools', array($this, 'display_tool_links'));
	}

	// Function to display the table with useful tool links
	public function display_tool_links() {
		// Get the current language
		$CI = &get_instance();
		$current_language = $CI->config->item('language');
		$this->load_language($current_language);

		// Example data
		$tool_links = [
			['name' => 'Free Modern SVG Illustrations', 'link_text' => 'Undraw illustrations', 'link_url' => 'https://undraw.co/illustrations'],
			['name' => 'Gradient Generator', 'link_text' => 'CSS Gradients colorbeta', 'link_url' => 'https://colorbeta.com/'],
			['name' => 'SVG Generator', 'link_text' => 'SVG Wave', 'link_url' => 'https://svgwave.in/'],
			['name' => 'SVG and Gradient Generator', 'link_text' => 'SVG bgjar', 'link_url' => 'https://bgjar.com/'],
			['name' => 'Animated Gradient Generator', 'link_text' => 'Animated Gradient background', 'link_url' => 'https://animated-gradient-background-generator.netlify.app/'],
		];

		// Generate the HTML table with reduced spacing
		$output = '<div class="col-md-12">';
		$output .= '<div class="card shadow mb-4">';
		$output .= '<div class="card-header py-3 bg-primary">';
		$output .= '<h6 class="m-0 font-weight-bold text-white"><i class="fas fa-link"></i> ' . ($this->lang['useful_tool_links_title'] ?? 'Useful Tool Links') . '</h6>';
		$output .= '</div>';
		$output .= '<div class="card-body">';
		$output .= '<table id="tool_links_table" class="table table-striped table-bordered" style="margin: 0; padding: 0;">';
		$output .= '<thead><tr><th style="padding: 4px;">' . ($this->lang['useful_tool_links_name'] ?? 'Name') . '</th><th style="padding: 4px;">' . ($this->lang['useful_tool_links_link'] ?? 'Link') . '</th></tr></thead>';
		$output .= '<tbody>';
		foreach ($tool_links as $tool) {
			$output .= '<tr>';
			$output .= '<td style="padding: 4px;">' . htmlspecialchars($tool['name'], ENT_QUOTES, 'UTF-8') . '</td>';
			$output .= '<td style="padding: 4px;"><a href="' . htmlspecialchars($tool['link_url'], ENT_QUOTES, 'UTF-8') . '" target="_blank">' . htmlspecialchars($tool['link_text'], ENT_QUOTES, 'UTF-8') . '</a></td>';
			$output .= '</tr>';
		}
		$output .= '</tbody>';
		$output .= '</table>';
		$output .= '</div>';
		$output .= '</div>';
		$output .= '</div>';

		// Include DataTables initialization script with DOM ready check and reinitialization prevention
		$output .= '
<script>
    document.addEventListener("DOMContentLoaded", function() {
        if ($.fn.DataTable.isDataTable("#tool_links_table")) {
            $("#tool_links_table").DataTable().destroy();
        }
        $("#tool_links_table").DataTable({
            paging: true,       // Enable pagination
            pageLength: 5,      // Set the number of entries per page to 5
            searching: true,    // Enable searching
            ordering: true,     // Enable column ordering
            info: true          // Show information about the table (e.g., "Showing 1 to 5 of 10 entries")
        });
    });
</script>';

		return $output;
	}
}
?>
